@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -20,18 +20,21 @@
  *
  */
 
+#include <QFile>
+#include <QDataStream>
+
 #include <core/Basics/Drumkit.h>
 #include <core/config.h>
 #ifdef H2CORE_HAVE_LIBARCHIVE
 #include <archive.h>
 #include <archive_entry.h>
 #else
-#ifndef WIN32
+  #ifndef WIN32
 #include <fcntl.h>
 #include <errno.h>
 #include <zlib.h>
 #include <libtar.h>
-#endif
+  #endif
 #endif
 
 #include <core/Basics/Sample.h>
@@ -41,8 +44,9 @@
 #include <core/Basics/InstrumentComponent.h>
 #include <core/Basics/InstrumentLayer.h>
 
-#include <core/Helpers/Xml.h>
+#include <core/Helpers/Future.h>
 #include <core/Helpers/Legacy.h>
+#include <core/Helpers/Xml.h>
 
 #include <core/Hydrogen.h>
 #include <core/SoundLibrary/SoundLibraryDatabase.h>
@@ -58,6 +62,8 @@ Drumkit::Drumkit() : __samples_loaded( false ),
 					 __license( License() ),
 					 __imageLicense( License() )
 {
+	QDir usrDrumkitPath( Filesystem::usr_drumkits_dir() );
+	__path = usrDrumkitPath.filePath( __name );
 	__components = std::make_shared<std::vector<std::shared_ptr<DrumkitComponent>>>();
 	__instruments = std::make_shared<InstrumentList>();
 }
@@ -85,7 +91,10 @@ Drumkit::~Drumkit()
 {
 }
 
-std::shared_ptr<Drumkit> Drumkit::load( const QString& sDrumkitPath, bool bUpgrade, bool bSilent )
+std::shared_ptr<Drumkit> Drumkit::load( const QString& sDrumkitPath,
+										bool bUpgrade,
+										bool* pLegacyFormatEncountered,
+										bool bSilent )
 {
 	if ( ! Filesystem::drumkit_valid( sDrumkitPath ) ) {
 		ERRORLOG( QString( "[%1] is not valid drumkit folder" ).arg( sDrumkitPath ) );
@@ -94,49 +103,64 @@ std::shared_ptr<Drumkit> Drumkit::load( const QString& sDrumkitPath, bool bUpgra
 
 	QString sDrumkitFile = Filesystem::drumkit_file( sDrumkitPath );
 	
-	bool bReadingSuccessful = true;
-	
 	XMLDoc doc;
-	if ( !doc.read( sDrumkitFile, Filesystem::drumkit_xsd_path(), true ) ) {
-		// Drumkit does not comply with the XSD schema
-		// definition. It's probably an old one. load_from() will try
-		// to handle it regardlessly but we should upgrade it in order
-		// to avoid this in future loads.
-		doc.read( sDrumkitFile, nullptr, bSilent );
-		
-		bReadingSuccessful = false;
-	}
-	
+	doc.read( sDrumkitFile, bSilent );
+
 	XMLNode root = doc.firstChildElement( "drumkit_info" );
 	if ( root.isNull() ) {
 		ERRORLOG( "drumkit_info node not found" );
 		return nullptr;
 	}
 
-	auto pDrumkit =
-		Drumkit::load_from( &root, sDrumkitFile.left( sDrumkitFile.lastIndexOf( "/" ) ),
-							bSilent );
-	
+	bool bLegacyFormatEncountered = false;
+
+	std::shared_ptr<Drumkit> pDrumkit = nullptr;
+
+	const QString sDrumkitDir =
+		sDrumkitFile.left( sDrumkitFile.lastIndexOf( "/" ) );
+
+	// Check whether the file was created using a newer version of Hydrogen.
+	auto formatVersionNode = root.firstChildElement( "formatVersion" );
+	if ( ! formatVersionNode.isNull() ) {
+		WARNINGLOG( QString( "Drumkit [%1] was created with a more recent version of Hydrogen than the current one!" )
+					.arg( sDrumkitPath ) );
+		// Even in case the future version is invalid with respect to the XSD
+		// file, the most recent version of the format will be the most
+		// successful one.
+		pDrumkit = Future::loadDrumkit( root, sDrumkitDir, bSilent );
+	}
+	else {
+		pDrumkit = Drumkit::load_from( &root, sDrumkitDir,
+									   &bLegacyFormatEncountered, bSilent );
+	}
+
+	if ( pLegacyFormatEncountered != nullptr ) {
+		*pLegacyFormatEncountered = bLegacyFormatEncountered;
+	}
+
 	if ( pDrumkit == nullptr ) {
 		ERRORLOG( QString( "Unable to load drumkit [%1]" ).arg( sDrumkitFile ) );
 		return nullptr;
 	}
 	
-	if ( ! bReadingSuccessful && bUpgrade ) {
+	if ( bLegacyFormatEncountered && bUpgrade ) {
 		upgrade_drumkit( pDrumkit, sDrumkitPath );
 	}
 	
 	return pDrumkit;
 }
 
-std::shared_ptr<Drumkit> Drumkit::load_from( XMLNode* node, const QString& sDrumkitPath, bool bSilent )
+std::shared_ptr<Drumkit> Drumkit::load_from( XMLNode* node,
+											 const QString& sDrumkitPath,
+											 bool* pLegacyFormatEncountered,
+											 bool bSilent )
 {
 	QString sDrumkitName = node->read_string( "name", "", false, false, bSilent );
 	if ( sDrumkitName.isEmpty() ) {
 		ERRORLOG( "Drumkit has no name, abort" );
 		return nullptr;
 	}
-	
+
 	std::shared_ptr<Drumkit> pDrumkit = std::make_shared<Drumkit>();
 
 	pDrumkit->__path = sDrumkitPath;
@@ -165,7 +189,8 @@ std::shared_ptr<Drumkit> Drumkit::load_from( XMLNode* node, const QString& sDrum
 	if ( ! componentListNode.isNull() ) {
 		XMLNode componentNode = componentListNode.firstChildElement( "drumkitComponent" );
 		while ( ! componentNode.isNull()  ) {
-			auto pDrumkitComponent = DrumkitComponent::load_from( &componentNode );
+			auto pDrumkitComponent = DrumkitComponent::load_from(
+				&componentNode, pLegacyFormatEncountered );
 			if ( pDrumkitComponent != nullptr ) {
 				pDrumkit->get_components()->push_back(pDrumkitComponent);
 			}
@@ -176,16 +201,23 @@ std::shared_ptr<Drumkit> Drumkit::load_from( XMLNode* node, const QString& sDrum
 		WARNINGLOG( "componentList node not found" );
 		auto pDrumkitComponent = std::make_shared<DrumkitComponent>( 0, "Main" );
 		pDrumkit->get_components()->push_back(pDrumkitComponent);
+
+		if ( pLegacyFormatEncountered != nullptr ) {
+			*pLegacyFormatEncountered = true;
+		}
 	}
 
-	auto pInstrumentList = InstrumentList::load_from( node,
-													  sDrumkitPath,
-													  sDrumkitName,
-													  license, false );
+	auto pInstrumentList = InstrumentList::load_from(
+		node, sDrumkitPath, sDrumkitName, license, pLegacyFormatEncountered,
+		false );
 	// Required to assure backward compatibility.
 	if ( pInstrumentList == nullptr ) {
 		WARNINGLOG( "instrument list could not be loaded. Using empty one." );
 		pInstrumentList = std::make_shared<InstrumentList>();
+
+		if ( pLegacyFormatEncountered != nullptr ) {
+			*pLegacyFormatEncountered = true;
+		}
 	}
 		
 	pDrumkit->set_instruments( pInstrumentList );
@@ -239,19 +271,12 @@ bool Drumkit::loadDoc( const QString& sDrumkitDir, XMLDoc* pDoc, bool bSilent )
 
 	const QString sDrumkitPath = Filesystem::drumkit_file( sDrumkitDir );
 
-	if( ! pDoc->read( sDrumkitPath, Filesystem::drumkit_xsd_path(), true ) ) {
-		if ( ! bSilent ) {
-			WARNINGLOG( QString( "[%1] does not validate against drumkit schema. Trying to retrieve its name nevertheless.")
-						.arg( sDrumkitPath ) );
-		}
-		
-		if ( ! pDoc->read( sDrumkitPath, nullptr, bSilent ) ) {
-			ERRORLOG( QString( "Unable to load drumkit name for [%1]" )
-					  .arg( sDrumkitPath ) );
-			return false;
-		}
+	if ( ! pDoc->read( sDrumkitPath, bSilent ) ) {
+		ERRORLOG( QString( "Unable to load drumkit name for [%1]" )
+				  .arg( sDrumkitPath ) );
+		return false;
 	}
-	
+
 	XMLNode root = pDoc->firstChildElement( "drumkit_info" );
 	if ( root.isNull() ) {
 		ERRORLOG( QString( "Unable to load drumkit name for [%1]. 'drumkit_info' node not found" )
@@ -461,7 +486,9 @@ bool Drumkit::save_samples( const QString& sDrumkitFolder, bool bSilent ) const
 	for ( int i = 0; i < pInstrList->size(); i++ ) {
 		auto pInstrument = ( *pInstrList )[i];
 		for ( const auto& pComponent : *pInstrument->get_components() ) {
-
+			if ( pComponent == nullptr ) {
+				continue;
+			}
 			for ( int n = 0; n < InstrumentComponent::getMaxLayers(); n++ ) {
 				auto pLayer = pComponent->get_layer( n );
 				if ( pLayer != nullptr && pLayer->get_sample() != nullptr ) {
@@ -491,11 +518,11 @@ bool Drumkit::save_samples( const QString& sDrumkitFolder, bool bSilent ) const
 	return true;
 }
 
-bool Drumkit::save_image( const QString& dk_dir, bool bSilent ) const
+bool Drumkit::save_image( const QString& sDrumkitDir, bool bSilent ) const
 {
-	if ( __image.length() > 0 ) {
+	if ( ! __image.isEmpty() && sDrumkitDir != __path ) {
 		QString src = __path + "/" + __image;
-		QString dst = dk_dir + "/" + __image;
+		QString dst = sDrumkitDir + "/" + __image;
 		if ( Filesystem::file_exists( src, bSilent ) ) {
 			if ( ! Filesystem::file_copy( src, dst, bSilent ) ) {
 				ERRORLOG( QString( "Error copying %1 to %2").arg( src ).arg( dst ) );
@@ -506,6 +533,137 @@ bool Drumkit::save_image( const QString& dk_dir, bool bSilent ) const
 	return true;
 }
 
+std::shared_ptr<DrumkitComponent> Drumkit::getComponent( int nID ) const
+{
+	for ( const auto& pComponent : *__components ) {
+		if ( pComponent->get_id() == nID ) {
+			return pComponent;
+		}
+	}
+
+	return nullptr;
+}
+
+int Drumkit::findUnusedComponentId() const {
+	int nNewId = __components->size();
+	for ( int ii = 0; ii < __components->size(); ++ii ) {
+		bool bIsPresent = false;
+		for ( const auto& ppComp : *__components ) {
+			if ( ppComp != nullptr && ppComp->get_id() == ii ) {
+				bIsPresent = true;
+				break;
+			}
+		}
+
+		if ( ! bIsPresent ){
+			nNewId = ii;
+			break;
+		}
+	}
+
+	return nNewId;
+}
+
+void Drumkit::addComponent( std::shared_ptr<DrumkitComponent> pComponent ) {
+	// Sanity check
+	if ( pComponent == nullptr ) {
+		ERRORLOG( "Invalid component" );
+		return;
+	}
+
+	for ( const auto& ppComponent : *__components ) {
+		if ( ppComponent == pComponent ) {
+			ERRORLOG( "Component is already present" );
+			return;
+		}
+	}
+
+	__components->push_back( pComponent );
+
+	for ( auto& ppInstrument : *__instruments ) {
+		ppInstrument->get_components()->push_back(
+			std::make_shared<InstrumentComponent>(pComponent->get_id()) );
+	}
+}
+
+void Drumkit::addInstrument( std::shared_ptr<Instrument> pInstrument ) {
+	if ( pInstrument == nullptr ) {
+		ERRORLOG( "invalid instrument" );
+		return;
+	}
+
+	// Ensure instrument components are contained in the Drumkit (compared by
+	// name). If not, add them.
+	for ( const auto& ppInstrumentComponent : *pInstrument->get_components() ) {
+		if ( ppInstrumentComponent == nullptr ) {
+			continue;
+		}
+
+		const int nComponentId = ppInstrumentComponent->get_drumkit_componentID();
+		if ( getComponent( nComponentId ) == nullptr ) {
+			ERRORLOG( QString( "No component of id [%1] found! Creating a new one" )
+					  .arg( nComponentId ) );
+
+			addComponent( std::make_shared<DrumkitComponent>(
+							  nComponentId, QString::number( nComponentId ) ) );
+		}
+	}
+
+	// Add components of this drumkit not already present in the instrument.
+	for ( const auto& ppThisKitsComponent : *__components ) {
+		if ( ppThisKitsComponent != nullptr ) {
+			bool bIsPresent = false;
+			for ( const auto& ppInstrumentCompnent : *pInstrument->get_components() ) {
+				if ( ppInstrumentCompnent != nullptr &&
+					 ppInstrumentCompnent->get_drumkit_componentID() ==
+					 ppThisKitsComponent->get_id() ) {
+					bIsPresent = true;
+					break;
+				}
+			}
+
+			if ( ! bIsPresent ){
+				auto pNewInstrCompo = std::make_shared<InstrumentComponent>(
+					ppThisKitsComponent->get_id() );
+				pInstrument->get_components()->push_back( pNewInstrCompo );
+			}
+		}
+	}
+
+	// In case there already is an instrument featuring its ID, we have to
+	// create a new one.
+	bool bIdAlreadyPresent = false;
+	for ( const auto& ppInstrument : *__instruments ) {
+		if ( ppInstrument != nullptr &&
+			 ppInstrument->get_id() == pInstrument->get_id() ) {
+			bIdAlreadyPresent = true;
+			break;
+		}
+	}
+	if ( bIdAlreadyPresent && pInstrument->get_id() >= 0 ) {
+		int nNewId = __instruments->size();
+		for ( int ii = 0; ii < __instruments->size(); ++ii ) {
+			bool bIsPresent = false;
+			for ( const auto& ppInstrument : *__instruments ) {
+				if ( ppInstrument != nullptr &&
+					 ppInstrument->get_id() == ii ) {
+					bIsPresent = true;
+					break;
+				}
+			}
+
+			if ( ! bIsPresent ) {
+				nNewId = ii;
+				break;
+			}
+		}
+
+		pInstrument->set_id( nNewId );
+	}
+
+	__instruments->add( pInstrument );
+}
+
 void Drumkit::set_instruments( std::shared_ptr<InstrumentList> instruments )
 {
 	__instruments = instruments;
@@ -560,11 +718,21 @@ bool Drumkit::remove( const QString& sDrumkitDir )
 	return true;
 }
 	
-bool Drumkit::install( const QString& sSourcePath, const QString& sTargetPath, bool bSilent )
+bool Drumkit::install( const QString& sSourcePath, const QString& sTargetPath,
+					   QString* pInstalledPath, bool* pEncodingIssuesDetected,
+					   bool bSilent )
 {
+	// Ensure variables are always set/initialized.
+	if ( pInstalledPath != nullptr ) {
+		*pInstalledPath = "";
+	}
+	if ( pEncodingIssuesDetected != nullptr ) {
+		*pEncodingIssuesDetected = false;
+	}
+
 	if ( sTargetPath.isEmpty() ) {
 		if ( ! bSilent ) {
-			_INFOLOG( QString( "Install drumkit [%1]" ).arg( sSourcePath ) );
+			INFOLOG( QString( "Install drumkit [%1]" ).arg( sSourcePath ) );
 		}
 		
 	} else {
@@ -573,88 +741,168 @@ bool Drumkit::install( const QString& sSourcePath, const QString& sTargetPath, b
 		}
 		
 		if ( ! bSilent ) {		
-			_INFOLOG( QString( "Extract drumkit from [%1] to [%2]" )
+			INFOLOG( QString( "Extract drumkit from [%1] to [%2]" )
 					  .arg( sSourcePath ).arg( sTargetPath ) );
 		}
 	}
 	
 #ifdef H2CORE_HAVE_LIBARCHIVE
-	int r;
-	struct archive* arch;
+	int nRet;
+
+	bool bUseUtf8Encoding = true;
+	if ( nullptr == setlocale( LC_ALL, "en_US.UTF-8" ) ) {
+		INFOLOG( "No en_US.UTF-8 locale not available on this system" );
+		bUseUtf8Encoding = false;
+	}
+
+	struct archive* a;
 	struct archive_entry* entry;
 
-	arch = archive_read_new();
-
-#if ARCHIVE_VERSION_NUMBER < 3000000
-	archive_read_support_compression_all( arch );
-#else
-	archive_read_support_filter_all( arch );
-#endif
-
-	archive_read_support_format_all( arch );
-
-#if ARCHIVE_VERSION_NUMBER < 3000000
-	if ( archive_read_open_file( arch, sSourcePath.toLocal8Bit(), 10240 ) ) {
-#else
-	if ( archive_read_open_filename( arch, sSourcePath.toLocal8Bit(), 10240 ) ) {
-#endif
-		_ERRORLOG( QString( "archive_read_open_file() [%1] %2" )
-				   .arg( archive_errno( arch ) )
-				   .arg( archive_error_string( arch ) ) );
-		archive_read_close( arch );
-
-#if ARCHIVE_VERSION_NUMBER < 3000000
-		archive_read_finish( arch );
-#else
-		archive_read_free( arch );
-#endif
+	if ( ! bSilent ) {
+		INFOLOG( QString( "Importing using `libarchive` version [%1]" )
+				 .arg( ARCHIVE_VERSION_STRING ) );
+	}
 
+	a = archive_read_new();
+	if ( a == nullptr ) {
+		ERRORLOG( "Unable to create new archive" );
 		return false;
 	}
-	bool ret = true;
-
-	QString dk_dir;
-	if ( ! sTargetPath.isEmpty() ) {
-		dk_dir = sTargetPath + "/";
-	} else {
-		dk_dir = Filesystem::usr_drumkits_dir() + "/";
-	}
-		
-	while ( ( r = archive_read_next_header( arch, &entry ) ) != ARCHIVE_EOF ) {
-		if ( r != ARCHIVE_OK ) {
-			_ERRORLOG( QString( "archive_read_next_header() [%1] %2" )
-					   .arg( archive_errno( arch ) )
-					   .arg( archive_error_string( arch ) ) );
-			ret = false;
-			break;
-		}
-		QString np = dk_dir + archive_entry_pathname( entry );
-
-		QByteArray newpath = np.toLocal8Bit();
-
-		archive_entry_set_pathname( entry, newpath.data() );
-		r = archive_read_extract( arch, entry, 0 );
-		if ( r == ARCHIVE_WARN ) {
-			_WARNINGLOG( QString( "archive_read_extract() [%1] %2" )
-						 .arg( archive_errno( arch ) )
-						 .arg( archive_error_string( arch ) ) );
-		} else if ( r != ARCHIVE_OK ) {
-			_ERRORLOG( QString( "archive_read_extract() [%1] %2" )
-					   .arg( archive_errno( arch ) )
-					   .arg( archive_error_string( arch ) ) );
-			ret = false;
-			break;
-		}
-	}
-	archive_read_close( arch );
 
 #if ARCHIVE_VERSION_NUMBER < 3000000
-	archive_read_finish( arch );
+	archive_read_support_compression_all( a );
 #else
-	archive_read_free( arch );
+	nRet = archive_read_support_filter_all( a );
+	if ( nRet != ARCHIVE_OK ) {
+		WARNINGLOG( QString("Couldn't add support for all filters: %1" )
+				  .arg( archive_error_string( a ) ) );
+	}
 #endif
 
-	return ret;
+	nRet = archive_read_support_format_all( a );
+	if ( nRet != ARCHIVE_OK ) {
+		WARNINGLOG( QString("Couldn't add support for all formats: %1" )
+				  .arg( archive_error_string( a ) ) );
+	}
+
+	// Shutdown version used on error. Therefore, contained commands are not
+	// checked for errors themselves.
+	auto tearDown = [&]() {
+		archive_read_close( a );
+
+#if ARCHIVE_VERSION_NUMBER < 3000000
+		archive_read_finish( a );
+#else
+		archive_read_free( a );
+#endif
+	};
+
+#if ARCHIVE_VERSION_NUMBER < 3000000
+	const auto sSourcePathUtf8 = sSourcePath.toUtf8();
+	nRet = archive_read_open_file( a, sSourcePathUtf8.constData(), 10240 );
+#else
+  #ifdef WIN32
+	QString sSourcePathPadded = sSourcePath;
+	sSourcePathPadded.append( '\0' );
+	auto sourcePathW = sSourcePathPadded.toStdWString();
+	nRet = archive_read_open_filename_w( a, sourcePathW.c_str(), 10240 );
+  #else
+	const auto sSourcePathUtf8 = sSourcePath.toUtf8();
+	nRet = archive_read_open_filename( a, sSourcePathUtf8.constData(),
+									   10240 );
+  #endif
+#endif
+	if ( nRet != ARCHIVE_OK ) {
+		ERRORLOG( QString( "Unable to open archive [%1] for reading: %2" )
+				   .arg( sSourcePath )
+				   .arg( archive_error_string( a ) ) );
+		tearDown();
+		return false;
+	}
+
+	QString sDrumkitDir;
+	if ( ! sTargetPath.isEmpty() ) {
+		sDrumkitDir = sTargetPath + "/";
+	} else {
+		sDrumkitDir = Filesystem::usr_drumkits_dir() + "/";
+	}
+		
+	while ( ( nRet = archive_read_next_header( a, &entry ) ) != ARCHIVE_EOF ) {
+		if ( nRet != ARCHIVE_OK ) {
+			ERRORLOG( QString( "Unable to read next archive header: %1" )
+					   .arg( archive_error_string( a ) ) );
+			tearDown();
+			return false;
+		}
+		if ( entry == nullptr ) {
+			ERRORLOG( "Couldn't read in next archive entry" );
+			return false;
+		}
+
+		QString sNewPath = QString::fromUtf8( archive_entry_pathname_utf8( entry ) );
+		if ( sNewPath.isEmpty() ) {
+			sNewPath = QString( archive_entry_pathname( entry ) );
+		}
+
+		if ( ! bUseUtf8Encoding ) {
+			// In case `libarchive` is not able to support UTF-8 on the system,
+			// we remove (a lot of) characters. Else they will be represented by
+			// wacky ones and the calling routine would have no idea where the
+			// resulting kit did end up.
+			const auto sNewPathTrimmed = Filesystem::removeUtf8Characters( sNewPath );
+			if ( sNewPathTrimmed != sNewPath ) {
+				ERRORLOG( QString( "Encoding error (no UTF-8 available)! File was renamed [%1] -> [%2]" )
+						  .arg( sNewPath ).arg( sNewPathTrimmed ) );
+				if ( pEncodingIssuesDetected != nullptr ) {
+					*pEncodingIssuesDetected = true;
+				}
+				sNewPath = sNewPathTrimmed;
+			}
+		}
+		sNewPath.prepend( sDrumkitDir );
+
+		if ( pInstalledPath != nullptr &&
+			 sNewPath.contains( Filesystem::drumkit_xml() ) ) {
+			// This file must be part of every kit and allows us to set this
+			// variable only once.
+			QFileInfo installInfo( sNewPath );
+			*pInstalledPath = installInfo.absoluteDir().absolutePath();
+		}
+		QByteArray newpath = sNewPath.toUtf8();
+
+		archive_entry_set_pathname( entry, newpath.data() );
+		nRet = archive_read_extract( a, entry, 0 );
+		if ( nRet == ARCHIVE_WARN ) {
+			WARNINGLOG( QString( "While extracting content of [%1] from archive: %2" )
+						 .arg( sNewPath )
+						 .arg( archive_error_string( a ) ) );
+		}
+		else if ( nRet != ARCHIVE_OK ) {
+			ERRORLOG( QString( "Unable to extract content of [%1] from archive: %2" )
+					   .arg( sNewPath )
+					   .arg( archive_error_string( a ) ) );
+			tearDown();
+			return false;
+		}
+	}
+	nRet = archive_read_close( a );
+	if ( nRet != ARCHIVE_OK ) {
+		ERRORLOG( QString("Couldn't close archive: %1" )
+				  .arg( archive_error_string( a ) ) );
+		return false;
+	}
+
+#if ARCHIVE_VERSION_NUMBER < 3000000
+	archive_read_finish( a );
+#else
+	nRet = archive_read_free( a );
+	if ( nRet != ARCHIVE_OK ) {
+		WARNINGLOG( QString("Couldn't free memory associated with archive: %1" )
+				  .arg( archive_error_string( a ) ) );
+	}
+#endif
+
+	return true;
 #else // H2CORE_HAVE_LIBARCHIVE
 #ifndef WIN32
 	// GUNZIP
@@ -713,7 +961,12 @@ bool Drumkit::install( const QString& sSourcePath, const QString& sTargetPath, b
 #endif
 }
 
-bool Drumkit::exportTo( const QString& sTargetDir, const QString& sComponentName, bool bRecentVersion, bool bSilent ) {
+bool Drumkit::exportTo( const QString& sTargetDir, const QString& sComponentName,
+						bool bRecentVersion, bool* pUtf8Encoded, bool bSilent ) {
+	if ( pUtf8Encoded != nullptr ) {
+		// Ensure the variable is always set/initialized.
+		*pUtf8Encoded = false;
+	}
 
 	if ( ! Filesystem::path_usable( sTargetDir, true, false ) ) {
 		ERRORLOG( QString( "Provided destination folder [%1] is not valid" )
@@ -825,10 +1078,10 @@ bool Drumkit::exportTo( const QString& sTargetDir, const QString& sComponentName
 	// The list does not have to be comprehensive as a "leakage" of
 	// audio files in the resulting .h2drumkit is not a big problem.
 	QStringList suffixBlacklist;
-	suffixBlacklist << "wav" << "flac" << "aifc" << "aif" << "aiff" << "au"
-					 << "caf" << "w64" << "ogg" << "pcm" << "l16" << "vob"
-					 << "mp1" << "mp2" << "mp3";
-	
+	for ( const auto& fformat : Filesystem::supportedAudioFormats() ) {
+		suffixBlacklist << Filesystem::AudioFormatToSuffix( fformat );
+	}
+
 	bool bSampleFound;
 	
 	for ( const auto& ssFile : sourceFilesList ) {
@@ -879,27 +1132,71 @@ bool Drumkit::exportTo( const QString& sTargetDir, const QString& sComponentName
 
 #if defined(H2CORE_HAVE_LIBARCHIVE)
 
+	if ( ! bSilent ) {
+		INFOLOG( QString( "Exporting using `libarchive` version [%1]" )
+				 .arg( ARCHIVE_VERSION_STRING ) );
+	}
+
+	bool bUseUtf8Encoding = true;
+	if ( nullptr == setlocale( LC_ALL, "en_US.UTF-8" ) ) {
+		ERRORLOG( "No en_US.UTF-8 locale not available on this system" );
+		bUseUtf8Encoding = false;
+	}
+
 	struct archive *a;
 	struct archive_entry *entry;
 	struct stat st;
-	char buff[8192];
-	int len;
-	FILE *f;
+	const int nBufferSize = 8192;
+	char buff[ nBufferSize ];
+	int nBytesRead, nRet;
+
+	// Write it back for the calling routine.
+	if ( pUtf8Encoded != nullptr ) {
+		*pUtf8Encoded = bUseUtf8Encoding;
+	}
 
 	a = archive_write_new();
+	if ( a == nullptr ) {
+		ERRORLOG( "Unable to create new archive" );
+		set_name( sOldDrumkitName );
+		return false;
+	}
 
-	#if ARCHIVE_VERSION_NUMBER < 3000000
-		archive_write_set_compression_gzip(a);
-	#else
-		archive_write_add_filter_gzip(a);
-	#endif
+#if ARCHIVE_VERSION_NUMBER < 3000000
+	archive_write_set_compression_gzip( a );
+#else
+	nRet = archive_write_add_filter_gzip( a );
+	if ( nRet != ARCHIVE_OK ) {
+		ERRORLOG( QString("Couldn't add GZIP filter: %1" )
+				  .arg( archive_error_string( a ) ) );
+		set_name( sOldDrumkitName );
+		return false;
+	}
+#endif
 
-	archive_write_set_format_pax_restricted(a);
-	
-	int ret = archive_write_open_filename(a, sTargetName.toUtf8().constData());
-	if ( ret != ARCHIVE_OK ) {
-		ERRORLOG( QString("Couldn't create archive [%0]" )
-			.arg( sTargetName ) );
+	nRet = archive_write_set_format_pax_restricted( a );
+	if ( nRet != ARCHIVE_OK ) {
+		ERRORLOG( QString("Couldn't set archive format to 'pax restricted': %1" )
+				  .arg( archive_error_string( a ) ) );
+		set_name( sOldDrumkitName );
+		return false;
+	}
+
+
+#ifdef WIN32
+	QString sTargetNamePadded = QString( sTargetName );
+	sTargetNamePadded.append( '\0' );
+	const auto targetPath = sTargetNamePadded.toStdWString();
+	nRet = archive_write_open_filename_w( a, targetPath.c_str() );
+#else
+	const auto sTargetNameUtf8 = sTargetName.toUtf8();
+	const auto targetPath = sTargetNameUtf8.constData();
+	nRet = archive_write_open_filename( a, targetPath );
+#endif
+	if ( nRet != ARCHIVE_OK ) {
+		ERRORLOG( QString("Couldn't create archive [%1]: %2" )
+				  .arg( targetPath )
+				  .arg( archive_error_string( a ) ) );
 		set_name( sOldDrumkitName );
 		return false;
 	}
@@ -919,29 +1216,83 @@ bool Drumkit::exportTo( const QString& sTargetDir, const QString& sComponentName
 			return false;
 		}
 
-		stat( sFilename.toUtf8().constData(), &st );
+		const auto sFilenameUtf8 = sFilename.toUtf8();
+		stat( sFilenameUtf8.constData(), &st );
 		entry = archive_entry_new();
-		archive_entry_set_pathname(entry, sTargetFilename.toUtf8().constData());
+		if ( entry == nullptr ) {
+			ERRORLOG( "Unable to create new archive entry" );
+			set_name( sOldDrumkitName );
+			return false;
+		}
+
+		const auto sTargetFilenameUtf8 = sTargetFilename.toUtf8();
+#if defined(WIN32) and ARCHIVE_VERSION_NUMBER >= 3005000
+		if ( bUseUtf8Encoding ) {
+			archive_entry_set_pathname_utf8(
+				entry, sTargetFilenameUtf8.constData());
+		} else {
+#else
+		{
+#endif
+			archive_entry_set_pathname(entry, sTargetFilenameUtf8.constData());
+		}
 		archive_entry_set_size(entry, st.st_size);
 		archive_entry_set_filetype(entry, AE_IFREG);
 		archive_entry_set_perm(entry, 0644);
-		archive_write_header(a, entry);
-		f = fopen( sFilename.toUtf8().constData(), "rb" );
-		len = fread(buff, sizeof(char), sizeof(buff), f);
-		while ( len > 0 ) {
-				archive_write_data(a, buff, len);
-				len = fread(buff, sizeof(char), sizeof(buff), f);
+		nRet = archive_write_header(a, entry);
+		if ( nRet != ARCHIVE_OK ) {
+			ERRORLOG( QString("Couldn't write entry for [%1] to archive header: %2" )
+					  .arg( sFilename )
+					  .arg( archive_error_string( a ) ) );
+			set_name( sOldDrumkitName );
+			return false;
 		}
-		fclose(f);
+
+		QFile file( sFilename );
+		if ( ! file.open( QIODevice::ReadOnly ) ) {
+			ERRORLOG( QString( "Unable to open file [%1] for reading" )
+				  .arg( sFilename ) );
+			archive_entry_free( entry );
+			continue;
+		}
+
+		QDataStream stream( &file );
+		nBytesRead = stream.readRawData( buff, nBufferSize );
+		while ( nBytesRead > 0 ) {
+			nRet = archive_write_data( a, buff, nBytesRead );
+			if ( nRet < 0 ) {
+				ERRORLOG( QString( "Error while writing data to entry of [%1]: %2" )
+						  .arg( sFilename ).arg( archive_error_string( a ) ) );
+				break;
+			}
+			else if ( nRet != nBytesRead ) {
+				WARNINGLOG( QString( "Only [%1/%2] bytes written to archive entry of [%3]" )
+							.arg( nRet ).arg( nBytesRead ).arg( sFilename ) );
+			}
+
+			nBytesRead = stream.readRawData( buff, nBufferSize );
+		}
+		file.close();
 		archive_entry_free(entry);
 	}
-	archive_write_close(a);
+	nRet = archive_write_close(a);
+	if ( nRet != ARCHIVE_OK ) {
+		ERRORLOG( QString("Couldn't close archive: %1" )
+				  .arg( archive_error_string( a ) ) );
+		set_name( sOldDrumkitName );
+		return false;
+	}
 
-	#if ARCHIVE_VERSION_NUMBER < 3000000
-		archive_write_finish(a);
-	#else
-		archive_write_free(a);
-	#endif
+
+#if ARCHIVE_VERSION_NUMBER < 3000000
+	archive_write_finish(a);
+#else
+	nRet = archive_write_free(a);
+	if ( nRet != ARCHIVE_OK ) {
+		WARNINGLOG( QString("Couldn't free memory associated with archive: %1" )
+				  .arg( archive_error_string( a ) ) );
+	}
+#endif
 
 	sourceFilesList.clear();
 
