@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -129,14 +129,17 @@ class Instrument : public H2Core::Object<Instrument>
 		 * load an instrument from an XMLNode
 		 * \param pNode the XMLDode to read from
 		 * \param sDrumkitPath the directory holding the drumkit
-		 * data. If empty, it will be read from @a pNode.
+		 *   data. If empty, it will be read from @a pNode.
 		 * \param sDrumkitName Name of the drumkit found in @a
-		 * sDrumkitPath.
+		 *   sDrumkitPath.
 		 * \param license License assigned to all Samples that will be
-		 * loaded. If empty, the license will be read from @a
-		 * sDrumkitPath.
+		 *   loaded. If empty, the license will be read from @a
+		 *   sDrumkitPath.
+		 * \param pLegacyFormatEncountered will be set to `true` is any of the
+		 *   XML elements requires legacy format support and left untouched
+		 *   otherwise.
 		 * \param bSilent if set to true, all log messages except of
-		 * errors and warnings are suppressed.
+		 *   errors and warnings are suppressed.
 		 *
 		 * \return a new Instrument instance
 		 */
@@ -144,6 +147,7 @@ class Instrument : public H2Core::Object<Instrument>
 													  const QString& sDrumkitPath = "",
 													  const QString& sDrumkitName = "",
 													  const License& license = License(),
+													  bool* pLegacyFormatEncountered = nullptr,
 													  bool bSilent = false );
 
 		///< set the name of the instrument
@@ -311,6 +315,12 @@ class Instrument : public H2Core::Object<Instrument>
 	/** Whether the instrument contains at least one non-missing
 	 * sample */
 	bool hasSamples() const;
+
+		/** Maximum support pitch value */
+		static constexpr float fPitchMax = 24.5;
+		/** Minimum support pitch value */
+		static constexpr float fPitchMin = -24.5;
+
 		/** Formatted string version for debugging purposes.
 		 * \param sPrefix String prefix which will be added in front of
 		 * every new line
@@ -572,11 +582,6 @@ inline void Instrument::set_random_pitch_factor( float val )
 	__random_pitch_factor = val;
 }
 
-inline void Instrument::set_pitch_offset( float val )
-{
-	__pitch_offset = val;
-}
-
 inline float Instrument::get_random_pitch_factor() const
 {
 	return __random_pitch_factor;
