@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -20,6 +20,8 @@
  *
  */
 
+#include <QDir>
+
 #include <core/AudioEngine/AudioEngine.h>
 #include <core/AudioEngine/TransportPosition.h>
 #include <core/CoreActionController.h>
@@ -92,6 +94,28 @@ bool CoreActionController::setStripVolume( int nStrip, float fVolumeValue, bool
 	return false;
 }
 
+bool CoreActionController::setInstrumentPitch( int nInstrument, float fValue ){
+	auto pSong = H2Core::Hydrogen::get_instance()->getSong();
+	if ( pSong == nullptr ) {
+		ERRORLOG( "no song set" );
+		return false;
+	}
+	auto pInstrumentList = pSong->getInstrumentList();
+	auto pInstrument = pInstrumentList->get( nInstrument );
+	if( pInstrument == nullptr ) {
+		ERRORLOG( QString( "Unable to retrieve instrument (Par. 1) [%1]" )
+				  .arg( nInstrument ) );
+		return false;
+	}
+
+	pInstrument->set_pitch_offset( fValue );
+	Hydrogen::get_instance()->setSelectedInstrumentNumber( nInstrument );
+	EventQueue::get_instance()->push_event( EVENT_INSTRUMENT_PARAMETERS_CHANGED,
+											nInstrument );
+
+	return true;
+}
+
 bool CoreActionController::setMetronomeIsActive( bool isActive )
 {
 	Preferences::get_instance()->m_bUseMetronome = isActive;
@@ -711,17 +735,8 @@ bool CoreActionController::savePreferences() {
 	return Preferences::get_instance()->savePreferences();
 }
 bool CoreActionController::quit() {
+	EventQueue::get_instance()->push_event( EVENT_QUIT, 0 );
 
-	if ( Hydrogen::get_instance()->getGUIState() != Hydrogen::GUIState::unavailable ) {
-		EventQueue::get_instance()->push_event( EVENT_QUIT, 0 );
-	} else {
-		// TODO: Close Hydrogen with no GUI present.
-		
-		ERRORLOG( QString( "Error: Closing the application via the core part is not supported yet!" ) );
-		return false;
-		
-	}
-	
 	return true;
 }
 
@@ -735,8 +750,9 @@ bool CoreActionController::activateTimeline( bool bActivate ) {
 
 	pHydrogen->setIsTimelineActivated( bActivate );
 	
-	if ( pHydrogen->getJackTimebaseState() == JackAudioDriver::Timebase::Slave ) {
-		WARNINGLOG( QString( "Timeline usage was [%1] in the Preferences. But these changes won't have an effect as long as there is still an external JACK timebase master." )
+	if ( pHydrogen->getJackTimebaseState() ==
+		 JackAudioDriver::Timebase::Listener ) {
+		WARNINGLOG( QString( "Timeline usage was [%1] in the Preferences. But these changes won't have an effect as long as there is still an external JACK Timebase controller." )
 					.arg( bActivate ? "enabled" : "disabled" ) );
 	} else if ( pHydrogen->getMode() == Song::Mode::Pattern ) {
 		WARNINGLOG( QString( "Timeline usage was [%1] in the Preferences. But these changes won't have an effect as long as Pattern Mode is still activated." )
@@ -854,31 +870,30 @@ bool CoreActionController::activateJackTransport( bool bActivate ) {
 #endif
 }
 
-bool CoreActionController::activateJackTimebaseMaster( bool bActivate ) {
+bool CoreActionController::activateJackTimebaseControl( bool bActivate ) {
 	auto pHydrogen = Hydrogen::get_instance();
 	
 #ifdef H2CORE_HAVE_JACK
 	if ( !pHydrogen->hasJackAudioDriver() ) {
-		ERRORLOG( "Unable to (de)activate Jack timebase master. Please select the Jack driver first." );
+		ERRORLOG( "Unable to (de)activate JACK Timebase support. Please select the JACK driver first." );
 		return false;
 	}
 	
 	pHydrogen->getAudioEngine()->lock( RIGHT_HERE );
 	if ( bActivate ) {
-		Preferences::get_instance()->m_bJackMasterMode = Preferences::USE_JACK_TIME_MASTER;
-		pHydrogen->onJackMaster();
+		Preferences::get_instance()->m_bJackTimebaseMode =
+			Preferences::USE_JACK_TIMEBASE_CONTROL;
+		pHydrogen->initJackTimebaseControl();
 	} else {
-		Preferences::get_instance()->m_bJackMasterMode = Preferences::NO_JACK_TIME_MASTER;
-		pHydrogen->offJackMaster();
+		Preferences::get_instance()->m_bJackTimebaseMode =
+			Preferences::NO_JACK_TIMEBASE_CONTROL;
+		pHydrogen->releaseJackTimebaseControl();
 	}
 	pHydrogen->getAudioEngine()->unlock();
 	
-	EventQueue::get_instance()->push_event( EVENT_JACK_TIMEBASE_STATE_CHANGED,
-											static_cast<int>(pHydrogen->getJackTimebaseState()) );
-	
 	return true;
 #else
-	ERRORLOG( "Unable to (de)activate Jack timebase master. Your Hydrogen version was not compiled with jack support." );
+	ERRORLOG( "Unable to (de)activate JACK Timebase support. Your Hydrogen version was not compiled with JACK support." );
 	return false;
 #endif
 }
@@ -911,7 +926,7 @@ bool CoreActionController::activateSongMode( bool bActivate ) {
 		pHydrogen->setMode( Song::Mode::Pattern );
 	}
 	
-	pAudioEngine->switchMode();
+	pAudioEngine->handleSongModeChanged();
 
 	pAudioEngine->unlock();
 	
@@ -950,6 +965,10 @@ bool CoreActionController::activateLoopMode( bool bActivate ) {
 		}
 		bChange = true;
 	}
+
+	pAudioEngine->lock( RIGHT_HERE );
+	pAudioEngine->handleLoopModeChanged();
+	pAudioEngine->unlock();
 	
 	if ( bChange ) {
 		EventQueue::get_instance()->push_event( EVENT_LOOP_MODE_ACTIVATION,
@@ -1054,9 +1073,10 @@ bool CoreActionController::upgradeDrumkit( const QString& sDrumkitPath, const QS
 
 	QString sTemporaryFolder, sDrumkitDir;
 	// Whether the drumkit was provided as compressed .h2drumkit file.
-	bool bIsCompressed;
-	auto pDrumkit = retrieveDrumkit( sDrumkitPath, &bIsCompressed,
-									 &sDrumkitDir, &sTemporaryFolder );
+	bool bIsCompressed, bLegacyFormatEncountered;
+	auto pDrumkit = retrieveDrumkit(
+		sDrumkitPath, &bIsCompressed, &sDrumkitDir, &sTemporaryFolder,
+		&bLegacyFormatEncountered );
 
 	if ( pDrumkit == nullptr ) {
 		ERRORLOG( QString( "Unable to load drumkit from source path [%1]" )
@@ -1132,7 +1152,7 @@ bool CoreActionController::upgradeDrumkit( const QString& sDrumkitPath, const QS
 			sExportPath = sourceFileInfo.dir().absolutePath();
 		}
 		
-		if ( ! pDrumkit->exportTo( sExportPath, "", true, false ) ) {
+		if ( ! pDrumkit->exportTo( sExportPath, "", true, nullptr, false ) ) {
 			ERRORLOG( QString( "Unable to export upgrade drumkit to [%1]" )
 					  .arg( sExportPath ) );
 			return false;
@@ -1160,9 +1180,10 @@ bool CoreActionController::validateDrumkit( const QString& sDrumkitPath, bool bC
 
 	QString sTemporaryFolder, sDrumkitDir;
 	// Whether the drumkit was provided as compressed .h2drumkit file.
-	bool bIsCompressed;
-	auto pDrumkit = retrieveDrumkit( sDrumkitPath, &bIsCompressed,
-									 &sDrumkitDir, &sTemporaryFolder );
+	bool bIsCompressed, bLegacyFormatEncountered;
+	const auto pDrumkit = retrieveDrumkit(
+		sDrumkitPath, &bIsCompressed, &sDrumkitDir, &sTemporaryFolder,
+		&bLegacyFormatEncountered );
 
 	if ( pDrumkit == nullptr ) {
 		ERRORLOG( QString( "Unable to load drumkit from source path [%1]" )
@@ -1176,53 +1197,24 @@ bool CoreActionController::validateDrumkit( const QString& sDrumkitPath, bool bC
 		return false;
 	}
 
-	auto validateXSD = [&]( const QString& sXSDPath, const QString& sContext ) {
-		
-		XMLDoc doc;
-		if ( !doc.read( Filesystem::drumkit_file( sDrumkitDir ),
-						sXSDPath, true ) ) {
-			ERRORLOG( QString( "Drumkit file [%1] does not comply with [%2] XSD definition" )
-					  .arg( Filesystem::drumkit_file( sDrumkitDir ) )
-					  .arg( sContext ) );
-			return false;
-		}
+	XMLDoc doc;
+	if ( !doc.read( Filesystem::drumkit_file( sDrumkitDir ), true ) ) {
+		ERRORLOG( QString( "Drumkit XML file [%1] can not be parsed." )
+				  .arg( Filesystem::drumkit_file( sDrumkitDir ) ) );
+		return false;
+	}
 	
-		XMLNode root = doc.firstChildElement( "drumkit_info" );
-		if ( root.isNull() ) {
-			ERRORLOG( QString( "Drumkit file [%1] seems bricked: 'drumkit_info' node not found" )
-					  .arg( Filesystem::drumkit_file( sDrumkitDir ) ) );
-			return false;
-		}
-
-		INFOLOG( QString( "Drumkit file [%1] validates [%2] XSD definition" )
-				 .arg( Filesystem::drumkit_file( sDrumkitDir ) )
-				 .arg( sContext ) );
-		
-		return true;
-	};
-
-	bool bValid = validateXSD( Filesystem::drumkit_xsd_path(), "current" );
-	if ( ! bValid && ! bCheckLegacyVersions ) {
+	XMLNode root = doc.firstChildElement( "drumkit_info" );
+	if ( root.isNull() ) {
+		ERRORLOG( QString( "Drumkit file [%1] seems bricked: 'drumkit_info' node not found" )
+				  .arg( Filesystem::drumkit_file( sDrumkitDir ) ) );
 		return false;
 	}
 
-	if ( ! bValid && bCheckLegacyVersions ) {
-		const auto legacyXSDFiles = Filesystem::drumkit_xsd_legacy_paths();
-
-		for ( const auto& sPath : legacyXSDFiles ) {
-			QString sContext( sPath );
-			sContext.remove( Filesystem::xsd_dir() );
-			sContext.remove( Filesystem::drumkit_xsd() );
-
-			if ( validateXSD( sPath, sContext ) ) {
-				bValid = true;
-				break;
-			}
-		}
-
-		if ( ! bValid ) {
-			return false;
-		}
+	if ( bLegacyFormatEncountered && ! bCheckLegacyVersions ) {
+		ERRORLOG( QString( "Drumkit [%1] uses a legacy format" )
+				  .arg( sDrumkitPath ) );
+		return false;
 	}
 
 	INFOLOG( QString( "Drumkit [%1] is valid!" )
@@ -1231,7 +1223,13 @@ bool CoreActionController::validateDrumkit( const QString& sDrumkitPath, bool bC
 	return true;
 }
 
-std::shared_ptr<Drumkit> CoreActionController::retrieveDrumkit( const QString& sDrumkitPath, bool* bIsCompressed, QString *sDrumkitDir, QString* sTemporaryFolder ) {
+std::shared_ptr<Drumkit> CoreActionController::retrieveDrumkit(
+	const QString& sDrumkitPath,
+	bool* bIsCompressed,
+	QString *sDrumkitDir,
+	QString* sTemporaryFolder,
+	bool* pLegacyFormatEncountered )
+{
 
 	std::shared_ptr<Drumkit> pDrumkit = nullptr;
 
@@ -1239,37 +1237,54 @@ std::shared_ptr<Drumkit> CoreActionController::retrieveDrumkit( const QString& s
 	// function is intended to be used for validating or upgrading
 	// drumkits via CLI or OSC command. It should always refer to the
 	// latest copy found on disk.
+	if ( bIsCompressed == nullptr || sTemporaryFolder == nullptr ||
+		 sDrumkitDir == nullptr || pLegacyFormatEncountered == nullptr ) {
+		ERRORLOG( "Invalid input" );
+		return nullptr;
+	}
 
 	*bIsCompressed = false;
 	*sTemporaryFolder = "";
 	*sDrumkitDir = "";
+	*pLegacyFormatEncountered = false;
 
 	QFileInfo sourceFileInfo( sDrumkitPath );
 
 	if ( Filesystem::dir_readable( sDrumkitPath, true ) ) {
 
 		// Providing the folder containing the drumkit
-		pDrumkit = Drumkit::load( sDrumkitPath, false, true );
+		pDrumkit = Drumkit::load(
+			sDrumkitPath, false, pLegacyFormatEncountered, true );
 		*sDrumkitDir = sDrumkitPath;
 		
-	} else if ( sourceFileInfo.fileName() == Filesystem::drumkit_xml() &&
-				Filesystem::file_readable( sDrumkitPath, true ) ) {
+	}
+	else if ( sourceFileInfo.fileName() == Filesystem::drumkit_xml() ) {
+		if ( ! Filesystem::file_readable( sDrumkitPath, true ) ) {
+			ERRORLOG( QString( "Drumkit file [%1] not readable" )
+					  .arg( sDrumkitPath ) );
+			return nullptr;
+		}
 
 		// Providing the path of a drumkit.xml file within a drumkit
 		// folder.
 		QString sDrumkitDirPath = QFileInfo( sDrumkitPath ).absoluteDir().absolutePath();
-		pDrumkit = Drumkit::load( sDrumkitDirPath, false, true );
+		pDrumkit = Drumkit::load(
+			sDrumkitDirPath, false, pLegacyFormatEncountered, true );
 		*sDrumkitDir = sourceFileInfo.dir().absolutePath();
 			
-	} else if ( ( "." + sourceFileInfo.suffix() ) == Filesystem::drumkit_ext &&
-				Filesystem::file_readable( sDrumkitPath, true ) ) {
+	}
+	else if ( ( "." + sourceFileInfo.suffix() ) == Filesystem::drumkit_ext ) {
+		if ( ! Filesystem::file_readable( sDrumkitPath, true ) ) {
+			ERRORLOG( QString( "Drumkit archive [%1] not readable" )
+					  .arg( sDrumkitPath ) );
+			return nullptr;
+		}
 
 		*bIsCompressed = true;
 		
 		// Temporary folder used to extract a compressed drumkit (
 		// .h2drumkit ).
-		QString sTemplateName( Filesystem::tmp_dir() + "/" +
-							   sourceFileInfo.baseName() + "_XXXXXX" );
+		QString sTemplateName( Filesystem::tmp_dir() + "/XXXXXX" );
 		QTemporaryDir tmpDir( sTemplateName );
 		tmpDir.setAutoRemove( false );
 		if ( ! tmpDir.isValid() ) {
@@ -1282,7 +1297,8 @@ std::shared_ptr<Drumkit> CoreActionController::retrieveDrumkit( const QString& s
 
 		// Providing the path to a compressed .h2drumkit file. It will
 		// be extracted to a temporary folder and loaded from there.
-		if ( ! Drumkit::install( sDrumkitPath, tmpDir.path(), true ) ) {
+		if ( ! Drumkit::install( sDrumkitPath, tmpDir.path(), sDrumkitDir,
+								 nullptr, true ) ) {
 			ERRORLOG( QString( "Unabled to extract provided drumkit [%1] into [%2]" )
 					  .arg( sDrumkitPath ).arg( tmpDir.path() ) );
 			return nullptr;
@@ -1310,9 +1326,8 @@ std::shared_ptr<Drumkit> CoreActionController::retrieveDrumkit( const QString& s
 			return nullptr;
 		}
 
-		*sDrumkitDir = tmpDir.path() + "/" + extractedFolders[0];
-		
-		pDrumkit = Drumkit::load( *sDrumkitDir, false, true );
+		pDrumkit = Drumkit::load(
+			*sDrumkitDir, false, pLegacyFormatEncountered, true );
 		
 	} else {
 		ERRORLOG( QString( "Provided source path [%1] does not point to a Hydrogen drumkit" )
@@ -1323,7 +1338,17 @@ std::shared_ptr<Drumkit> CoreActionController::retrieveDrumkit( const QString& s
 	return pDrumkit;
 }
 
-bool CoreActionController::extractDrumkit( const QString& sDrumkitPath, const QString& sTargetDir ) {
+bool CoreActionController::extractDrumkit( const QString& sDrumkitPath,
+										   const QString& sTargetDir,
+										   QString* pInstalledPath,
+										   bool* pEncodingIssuesDetected ) {
+	// Ensure variables are always set/initialized.
+	if ( pInstalledPath != nullptr ) {
+		*pInstalledPath = "";
+	}
+	if ( pEncodingIssuesDetected != nullptr ) {
+		*pEncodingIssuesDetected = false;
+	}
 
 	QString sTarget;
 	bool bInstall = false;
@@ -1351,7 +1376,8 @@ bool CoreActionController::extractDrumkit( const QString& sDrumkitPath, const QS
 		return false;
 	}
 
-	if ( ! Drumkit::install( sDrumkitPath, sTarget, true ) ) {
+	if ( ! Drumkit::install( sDrumkitPath, sTarget, pInstalledPath,
+							 pEncodingIssuesDetected, true ) ) {
 		ERRORLOG( QString( "Unabled to extract provided drumkit [%1] into [%2]" )
 				  .arg( sDrumkitPath ).arg( sTarget ) );
 		return false;
@@ -1583,6 +1609,44 @@ bool CoreActionController::removePattern( int nPatternNumber ) {
 	return true;
 }
 
+bool CoreActionController::clearInstrumentInPattern( int nInstrument,
+													 int nPatternNumber ) {
+
+	Hydrogen* pHydrogen = Hydrogen::get_instance();
+	auto pSong = pHydrogen->getSong();
+	if ( pSong == nullptr ) {
+		ERRORLOG( "no song set" );
+		return false;
+	}
+
+	int nPattern;
+	if ( nPatternNumber != -1 ) {
+		nPattern = nPatternNumber;
+	} else {
+		nPattern = pHydrogen->getSelectedPatternNumber();
+	}
+
+	auto pPattern = pSong->getPatternList()->get( nPattern );
+	if ( pPattern == nullptr ) {
+		ERRORLOG( QString( "Couldn't find pattern [%1]" ).arg( nPattern ) );
+		return false;
+	}
+
+	auto pInstrument = pSong->getInstrumentList()->get( nInstrument );
+	if ( pInstrument == nullptr ) {
+		ERRORLOG( QString( "Couldn't find instrument [%1]" ).arg( nInstrument ) );
+		return false;
+	}
+
+	pPattern->purge_instrument( pInstrument, true );
+
+	if ( pHydrogen->getGUIState() != Hydrogen::GUIState::unavailable ) {
+		EventQueue::get_instance()->push_event( EVENT_PATTERN_MODIFIED, 0 );
+	}
+
+	return true;
+}
+
 bool CoreActionController::toggleGridCell( int nColumn, int nRow ){
 	auto pHydrogen = Hydrogen::get_instance();
 
@@ -1592,6 +1656,7 @@ bool CoreActionController::toggleGridCell( int nColumn, int nRow ){
 	}
 	
 	auto pSong = pHydrogen->getSong();
+	auto pAudioEngine = pHydrogen->getAudioEngine();
 	auto pPatternList = pSong->getPatternList();
 	std::vector<PatternList*>* pColumns = pSong->getPatternGroupVector();
 
@@ -1609,7 +1674,7 @@ bool CoreActionController::toggleGridCell( int nColumn, int nRow ){
 		return false;
 	}
 
-	pHydrogen->getAudioEngine()->lock( RIGHT_HERE );
+	pAudioEngine->lock( RIGHT_HERE );
 	if ( nColumn >= 0 && nColumn < pColumns->size() ) {
 		PatternList *pColumn = ( *pColumns )[ nColumn ];
 		auto pPattern = pColumn->del( pNewPattern );
@@ -1630,7 +1695,8 @@ bool CoreActionController::toggleGridCell( int nColumn, int nRow ){
 				}
 			}
 		}
-	} else if ( nColumn >= pColumns->size() ) {
+	}
+	else if ( nColumn >= pColumns->size() ) {
 		// We need to add some new columns..
 		PatternList *pColumn;
 
@@ -1639,17 +1705,19 @@ bool CoreActionController::toggleGridCell( int nColumn, int nRow ){
 			pColumns->push_back( pColumn );
 		}
 		pColumn->add( pNewPattern );
-	} else {
+	}
+	else {
 		// nColumn < 0
 		ERRORLOG( QString( "Provided column [%1] is out of bound [0,%2]" )
 				  .arg( nColumn ).arg( pColumns->size() ) );
+		pAudioEngine->unlock();
 		return false;
 	}
 	
 	pHydrogen->updateSongSize();
 	pHydrogen->updateSelectedPattern( false );
 	
-	pHydrogen->getAudioEngine()->unlock();
+	pAudioEngine->unlock();
 
 	pHydrogen->setIsModified( true );
 	
@@ -1661,6 +1729,87 @@ bool CoreActionController::toggleGridCell( int nColumn, int nRow ){
 	return true;
 }
 
+bool CoreActionController::handleNote( int nNote, float fVelocity, bool bNoteOff ) {
+	auto pPref = Preferences::get_instance();
+	auto pHydrogen = Hydrogen::get_instance();
+	auto pSong = pHydrogen->getSong();
+	if ( pSong == nullptr ) {
+		ERRORLOG( "no song set" );
+		return false;
+	}
+
+	std::shared_ptr<Instrument> pInstrument = nullptr;
+	int nInstrument = 0;
+	QString sMode;
+
+	auto pInstrumentList = pSong->getInstrumentList();
+
+	if ( pPref->__playselectedinstrument ){
+		nInstrument = pHydrogen->getSelectedInstrumentNumber();
+		pInstrument = pInstrumentList->get( pHydrogen->getSelectedInstrumentNumber());
+		if ( pInstrument == nullptr ) {
+			WARNINGLOG( "No instrument selected!" );
+			return false;
+		}
+		sMode = "Play Selected Instrument";
+	}
+	else if ( pPref->m_bMidiFixedMapping ){
+		pInstrument = pInstrumentList->findMidiNote( nNote );
+		if ( pInstrument == nullptr ) {
+			WARNINGLOG( QString( "Unable to map note [%1] to instrument" )
+						.arg( nNote ) );
+			return false;
+		}
+		nInstrument = pInstrumentList->index( pInstrument );
+		sMode = "Map to Output MIDI note";
+	}
+	else {
+		nInstrument = nNote - MidiMessage::instrumentOffset;
+		if( nInstrument < 0 || nInstrument >= pInstrumentList->size()) {
+			WARNINGLOG( QString( "Instrument number [%1] - derived from note [%2] - out of bound note [%3,%4]" )
+						.arg( nInstrument ).arg( nNote )
+						.arg( 0 ).arg( pInstrumentList->size() ) );
+			return false;
+		}
+
+		pInstrument = pInstrumentList->get( nInstrument );
+		if ( pInstrument == nullptr ) {
+			WARNINGLOG( QString( "Unable to retrieve instrument [%1]" )
+						.arg( nInstrument ) );
+			return false;
+		}
+		sMode = "Map to instrument list position";
+	}
+
+
+	// Only look to change instrument if the current note is actually of hihat
+	// and hihat openness is outside the instrument selected
+	const int nHihatOpenness = pHydrogen->getHihatOpenness();
+	if ( pInstrument != nullptr &&
+		 pInstrument->get_hihat_grp() >= 0 &&
+		 ( nHihatOpenness < pInstrument->get_lower_cc() ||
+		   nHihatOpenness > pInstrument->get_higher_cc() ) ) {
+
+		for ( int i = 0; i <= pInstrumentList->size(); i++ ) {
+			auto ppInstr = pInstrumentList->get( i );
+			if ( ppInstr != nullptr &&
+				pInstrument->get_hihat_grp() == ppInstr->get_hihat_grp() &&
+				nHihatOpenness >= ppInstr->get_lower_cc() &&
+				nHihatOpenness <= ppInstr->get_higher_cc() ) {
+
+				nInstrument = i;
+				sMode = "Hihat Pressure Group";
+				break;
+			}
+		}
+	}
+
+	INFOLOG( QString( "[%1] mapped note [%2] to instrument [%3]" )
+			 .arg( sMode ).arg( nNote ).arg( nInstrument ) );
+
+	return pHydrogen->addRealtimeNote( nInstrument, fVelocity, bNoteOff, nNote );
+}
+
 void CoreActionController::updatePreferences() {
 	auto pPref = Preferences::get_instance();
 	auto pHydrogen = Hydrogen::get_instance();
@@ -1685,11 +1834,16 @@ void CoreActionController::insertRecentFile( const QString sFilename ){
 	bool bAlreadyContained = false;
 
 	std::vector<QString> recentFiles = pPref->getRecentFiles();
-	
-	recentFiles.insert( recentFiles.begin(), sFilename );
+
+	// We have to normalize directory separators. Else opening a
+	// song via double click from file browser and from within
+    // Hydrogen will give to distinct entries on Windows.
+    const QString sFilenameCleaned = QDir::cleanPath( sFilename );
+
+    recentFiles.insert( recentFiles.begin(), sFilenameCleaned );
 
 	if ( std::find( recentFiles.begin(), recentFiles.end(),
-					sFilename ) != recentFiles.end() ) {
+					sFilenameCleaned ) != recentFiles.end() ) {
 		// Eliminate all duplicates in the list while keeping the one
 		// inserted at the beginning. Also, in case the file got renamed,
 		// remove it's previous name from the list.
