@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -32,6 +32,7 @@
 #include <QtCore/QFileInfo>
 #include <QtCore/QCoreApplication>
 #include <QDateTime>
+#include <QRegularExpression>
 
 #ifdef H2CORE_HAVE_OSC
 #include <core/NsmClient.h>
@@ -116,8 +117,104 @@ QStringList Filesystem::__ladspa_paths;
 
 QString Filesystem::m_sPreferencesOverwritePath = "";
 
-/* TODO QCoreApplication is not instantiated */
-bool Filesystem::bootstrap( Logger* logger, const QString& sys_path )
+std::vector<Filesystem::AudioFormat> Filesystem::m_supportedAudioFormats = {
+	AudioFormat::Wav,
+	AudioFormat::Aif,
+	AudioFormat::Aifc,
+	AudioFormat::Aiff,
+	AudioFormat::Au,
+	AudioFormat::Caf,
+	AudioFormat::Voc,
+#ifdef H2CORE_HAVE_FLAC_SUPPORT
+	AudioFormat::Ogg,
+	AudioFormat::Flac,
+#endif
+#ifdef H2CORE_HAVE_OPUS_SUPPORT
+	AudioFormat::Opus,
+#endif
+#ifdef H2CORE_HAVE_MP3_SUPPORT
+	AudioFormat::Mp3,
+#endif
+	AudioFormat::W64
+};
+
+QString Filesystem::AudioFormatToSuffix( const AudioFormat& format, bool bSilent ) {
+	switch( format ) {
+		case AudioFormat::Aif:
+		case AudioFormat::Aifc:
+		case AudioFormat::Aiff:
+			return "aiff";
+		case AudioFormat::Au:
+			return "au";
+		case AudioFormat::Caf:
+			return "caf";
+		case AudioFormat::Flac:
+			return "flac";
+		case AudioFormat::Mp3:
+			return "mp3";
+		case AudioFormat::Ogg:
+			return "ogg";
+		case AudioFormat::Opus:
+			return "opus";
+		case AudioFormat::Voc:
+			return "voc";
+		case AudioFormat::W64:
+			return "w64";
+		case AudioFormat::Wav:
+			return "wav";
+		case AudioFormat::Unknown:
+		default:
+			if ( ! bSilent ) {
+				ERRORLOG( "Unknown audio format" );
+			}
+			return "";
+	}
+}
+
+Filesystem::AudioFormat Filesystem::AudioFormatFromSuffix( const QString& sPath,
+														   bool bSilent ) {
+	const QString sPathLower = sPath.toLower();
+	if ( sPathLower.endsWith( "aiff" ) ) {
+		return AudioFormat::Aif;
+	}
+	else if ( sPathLower.endsWith( "au" ) ) {
+		return AudioFormat::Au;
+	}
+	else if ( sPathLower.endsWith( "caf" ) ) {
+		return AudioFormat::Caf;
+	}
+	else if ( sPathLower.endsWith( "flac" ) ) {
+		return AudioFormat::Flac;
+	}
+	else if ( sPathLower.endsWith( "mp3" ) ) {
+		return AudioFormat::Mp3;
+	}
+	else if ( sPathLower.endsWith( "ogg" ) ) {
+		return AudioFormat::Ogg;
+	}
+	else if ( sPathLower.endsWith( "opus" ) ) {
+		return AudioFormat::Opus;
+	}
+	else if ( sPathLower.endsWith( "voc" ) ) {
+		return AudioFormat::Voc;
+	}
+	else if ( sPathLower.endsWith( "w64" ) ) {
+		return AudioFormat::W64;
+	}
+	else if ( sPathLower.endsWith( "wav" ) ) {
+		return AudioFormat::Wav;
+	} else {
+		if ( ! bSilent ) {
+			ERRORLOG( QString( "Unknown suffix in [%1]" ).arg( sPath ) );
+		}
+		return AudioFormat::Unknown;
+	}
+}
+
+bool Filesystem::bootstrap( Logger* logger, const QString& sSysDataPath,
+							const QString& sUsrDataPath,
+							const QString& sUserConfigPath,
+							const QString& sLogFile )
 {
 	if( __logger==nullptr && logger!=nullptr ) {
 		__logger = logger;
@@ -125,6 +222,9 @@ bool Filesystem::bootstrap( Logger* logger, const QString& sys_path )
 		return false;
 	}
 
+	// A QCoreApplication instance is needed for applicationDirPath etc.
+	assert( QCoreApplication::instance() != nullptr );
+
 #ifdef Q_OS_MACX
 #ifdef H2CORE_HAVE_BUNDLE
 	// Bundle: Prepare hydrogen to use path names which are used in app bundles: http://en.wikipedia.org/wiki/Application_Bundle
@@ -147,7 +247,37 @@ bool Filesystem::bootstrap( Logger* logger, const QString& sys_path )
 	__usr_data_path = QDir::homePath().append( "/" H2_USR_PATH "/data/" );
 	__usr_cfg_path = QDir::homePath().append( "/" H2_USR_PATH "/" USR_CONFIG );
 #endif
-	if( sys_path!=nullptr ) __sys_data_path = sys_path;
+	if ( ! sSysDataPath.isEmpty() ) {
+		INFOLOG( QString( "Using custom system data folder [%1]" )
+				 .arg( sSysDataPath ) );
+		__sys_data_path = sSysDataPath;
+		// Sanity check
+		if ( ! __sys_data_path.endsWith( QDir::separator() ) ) {
+			__sys_data_path.append( QDir::separator() );
+		}
+	}
+
+	if ( ! sUsrDataPath.isEmpty() ) {
+		INFOLOG( QString( "Using custom user data folder [%1]" )
+				 .arg( sUsrDataPath ) );
+		__usr_data_path = sUsrDataPath;
+		// Sanity check
+		if ( ! __usr_data_path.endsWith( QDir::separator() ) ) {
+			__usr_data_path.append( QDir::separator() );
+		}
+	}
+
+	if ( ! sUserConfigPath.isEmpty() ) {
+		INFOLOG( QString( "Using custom user-level config file [%1]" )
+				 .arg( sUserConfigPath ) );
+		__usr_cfg_path = sUserConfigPath;
+	}
+
+	if ( ! sLogFile.isEmpty() ) {
+		// No need for an info log. This is done within the bootstrap of the
+		// logger.
+		Filesystem::__usr_log_path = sLogFile;
+	}
 
 	if( !dir_readable( __sys_data_path ) ) {
 		__sys_data_path = QCoreApplication::applicationDirPath().append( "/" LOCAL_DATA_PATH );
@@ -289,9 +419,7 @@ bool Filesystem::path_usable( const QString& path, bool create, bool silent )
 			INFOLOG( QString( "create user directory : %1" ).arg( path ) );
 		}
 		if ( create && !QDir( "/" ).mkpath( path ) ) {
-			if( !silent ) {
-				ERRORLOG( QString( "unable to create user directory : %1" ).arg( path ) );
-			}
+			ERRORLOG( QString( "unable to create user directory : %1" ).arg( path ) );
 			return false;
 		}
 	}
@@ -309,7 +437,8 @@ bool Filesystem::write_to_file( const QString& dst, const QString& content )
 		ERRORLOG( QString( "unable to write to %1" ).arg( dst ) );
 		return false;
 	}
-	file.write( content.toUtf8().data() );
+	const auto contentUtf8 = content.toUtf8();
+	file.write( contentUtf8.data() );
 	file.close();
 
 	return true;
@@ -466,7 +595,11 @@ QString Filesystem::sys_config_path()
 }
 QString Filesystem::usr_config_path()
 {
-       return __usr_cfg_path;
+	if ( ! m_sPreferencesOverwritePath.isEmpty() ) {
+		return m_sPreferencesOverwritePath;
+	} else {
+		return __usr_cfg_path;
+	}
 }
 QString Filesystem::empty_sample_path()
 {
@@ -631,7 +764,8 @@ QString Filesystem::tmp_file_path( const QString &base )
 {
 	// Ensure template base will produce a valid filename
 	QString validBase = base;
-	validBase.remove( QRegExp( "[^a-zA-Z0-9._]" ) );
+	validBase.remove(
+		QRegularExpression( "[\\\\|\\/|\\*|\\,|\\$|:|=|@|!|\\^|&|\\?|\"|'|>|<|\\||%|:]+" ) );
 
 	QFileInfo f( validBase );
 	QString templateName( tmp_dir() + "/" );
@@ -697,46 +831,52 @@ QStringList Filesystem::usr_drumkit_list( )
 	return drumkit_list( usr_drumkits_dir() ) ;
 }
 
-QString Filesystem::prepare_sample_path( const QString& fname )
+QString Filesystem::prepare_sample_path( const QString& sSamplePath )
 {
-	int idx = get_basename_idx_under_drumkit( fname );
-	if ( idx >= 0 ) {
-		return fname.midRef( idx ).toString();
-	}
-	return fname;
-}
+	// Check whether the provided absolute sample path is located within a
+	// known drumkit directory.
+	int nIndexMatch = -1;
+	// On Windows the provided system dir needs cleaning and looks like this
+	// [C:\\projects\\hydrogen/data/\\drumkits/]. For all other OSs this is not
+	// necessary. But it does no harm either and might be a live safer in some
+	// edge cases.
+	const auto drumkitFolders = QStringList()
+		<< sys_drumkits_dir().replace( "\\", "/" ).replace( "//", "/" )
+		<< usr_drumkits_dir().replace( "\\", "/" ).replace( "//", "/" );
 
-bool Filesystem::file_is_under_drumkit( const QString& fname )
-{
-	return get_basename_idx_under_drumkit( fname ) != -1;
-}
+	QString sSamplePathCleaned( sSamplePath );
+#ifdef WIN32
+	// Qt uses posix separators `/` internally but things can easily mix up
+	// (maybe due to our code) and we end up with something like
+	// C:\projects\hydrogen/data/\drumkits/GMRockKit/Kick-Softest.wav .
+	// We have to ensure to work on a single separator.
+	sSamplePathCleaned = QString( sSamplePathCleaned ).replace( "\\", "/" );
+#endif
 
-int Filesystem::get_basename_idx_under_drumkit( const QString& fname )
-{
-	if( fname.startsWith( usr_drumkits_dir() ) )
-	{
-		int start = usr_drumkits_dir().size();
-		int index = fname.indexOf( "/", start );
-		QString dk_name = fname.midRef( start , index - start).toString();
-		if ( usr_drumkit_list().contains( dk_name ) ) {
-			return index + 1;
+	// When composing paths by combining different elements, two file separators
+	// can be used in a row. This is no problem in file access itself but would
+	// mess up our index-based approach in here.
+	sSamplePathCleaned = QString( sSamplePathCleaned ).replace( "//", "/" );
+
+	for ( const auto& ssFolder : drumkitFolders ) {
+		if ( sSamplePathCleaned.startsWith( ssFolder ) ) {
+			nIndexMatch = sSamplePathCleaned.indexOf(
+				"/", ssFolder.size() ) + 1;
+			break;
 		}
 	}
 
-	if( fname.startsWith( sys_drumkits_dir() ) )
-	{
-		int start = sys_drumkits_dir().size();
-		int index = fname.indexOf( "/", start);
-		QString dk_name = fname.midRef( start, index - start).toString();
-		if ( sys_drumkit_list().contains( dk_name ) ) {
-			return index + 1;
-		}
+	if ( nIndexMatch >= 0 ) {
+		// Sample is located in a drumkit folder. Just return basename.
+		QString sShortenedPath = sSamplePathCleaned.right(
+			sSamplePathCleaned.size() - nIndexMatch );
+
+		return std::move( sShortenedPath );
 	}
 
-	return -1;
+	return sSamplePath;
 }
 
-
 bool Filesystem::drumkit_exists( const QString& dk_name )
 {
 	if( usr_drumkit_list().contains( dk_name ) ) return true;
@@ -919,19 +1059,19 @@ bool Filesystem::isSongPathValid( const QString& sSongPath, bool bCheckExistance
 
 	if ( !songFileInfo.isAbsolute() ) {
 		ERRORLOG( QString( "Error: Unable to handle path [%1]. Please provide an absolute file path!" )
-						.arg( sSongPath.toLocal8Bit().data() ));
+						.arg( sSongPath ));
 		return false;
 	}
 	
 	if ( songFileInfo.exists() ) {
 		if ( !songFileInfo.isReadable() ) {
 			ERRORLOG( QString( "Unable to handle path [%1]. You must have permissions to read the file!" )
-						.arg( sSongPath.toLocal8Bit().data() ));
+						.arg( sSongPath ));
 			return false;
 		}
 		if ( !songFileInfo.isWritable() ) {
 			WARNINGLOG( QString( "You don't have permissions to write to the Song found in path [%1]. It will be opened as read-only (no autosave)." )
-						.arg( sSongPath.toLocal8Bit().data() ));
+						.arg( sSongPath ));
 			EventQueue::get_instance()->push_event( EVENT_UPDATE_SONG, 2 );
 		}
 	} else if ( bCheckExistance ) {
@@ -941,7 +1081,7 @@ bool Filesystem::isSongPathValid( const QString& sSongPath, bool bCheckExistance
 	
 	if ( songFileInfo.suffix() != "h2song" ) {
 		ERRORLOG( QString( "Unable to handle path [%1]. The provided file must have the suffix '.h2song'!" )
-					.arg( sSongPath.toLocal8Bit().data() ));
+					.arg( sSongPath ));
 		return false;
 	}
 	
@@ -953,7 +1093,8 @@ QString Filesystem::validateFilePath( const QString& sPath ) {
 	// Ensure the name will be a valid filename
 	QString sValidName( sPath );
 	sValidName.replace( " ", "_" );
-	sValidName.remove( QRegExp( "[^a-zA-Z0-9_-]" ) );
+	sValidName.remove(
+		QRegularExpression( "[\\\\|\\/|\\*|\\,|\\$|:|=|@|!|\\^|&|\\?|\"|'|>|<|\\||%|:]+" ) );
 
 	return sValidName;
 }
@@ -1120,6 +1261,15 @@ QString Filesystem::rerouteDrumkitPath( const QString& sDrumkitPath ) {
 	return sDrumkitPath;
 #endif
 }
+
+QString Filesystem::removeUtf8Characters( const QString &sEncodedString ) {
+	QString sCleaned( sEncodedString );
+	return sCleaned.remove(
+		QRegularExpression( "[^a-zA-Z0-9._/\\s()\\[\\]\\&\\+\\-]" ) );
+}
+const std::vector<Filesystem::AudioFormat>& Filesystem::supportedAudioFormats() {
+	return m_supportedAudioFormats;
+}
 };
 
 /* vim: set softtabstop=4 noexpandtab: */
