@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -64,14 +64,15 @@ public:
 		Song = 0,
 		/** Only tempo markers on the Timeline are considered.*/
 		Timeline = 1,
-		/** Hydrogen will disregard all internal tempo settings and
-			uses the ones provided by the JACK server instead. This
-			mode is only used in case the JACK audio driver is used,
-			JACK timebase support is activated in the Preferences, and
-			an external timebase master is registered to the JACK
-			server.*/
+		/** Hydrogen will disregard all internal tempo settings and uses the
+			ones provided by the JACK server instead. This mode is only used in
+			case the JACK audio driver is used, JACK Timebase support is
+			activated in the Preferences, and an external Timebase controller is
+			registered to the JACK server.*/
 		Jack = 2
 	};
+
+
 	/**
 	 * Creates all the instances used within Hydrogen in the right
 	 * order.
@@ -192,12 +193,14 @@ public:
 
 	void updateSongSize();
 
-		void			addRealtimeNote ( int instrument,
+		bool			addRealtimeNote ( int instrument,
 							  float velocity,
-							  float fPan = 0.0f,
 							  bool noteoff=false,
 							  int msg1=0 );
 
+		int getHihatOpenness() const;
+		void setHihatOpenness( int nValue );
+
 		void			restartDrivers();
 
 		AudioOutput*		getAudioOutput() const;
@@ -238,9 +241,8 @@ void			previewSample( Sample *pSample );
 
 	enum ErrorMessages {
 		/**
-		 * The provided input string in createDriver() does
-		 * not match any of the choices for
-		 * Preferences::m_sAudioDriver.
+		 * The provided input in createDriver() does not match any of the
+		 * choices for #H2Core::Preferences::AudioDriver.
 		 */
 		UNKNOWN_DRIVER,
 		/**
@@ -293,11 +295,16 @@ void			previewSample( Sample *pSample );
 	/**
 	 * Sets #m_nSelectedPatternNumber.
 	 *
-	 *\param nPat Sets #m_nSelectedPatternNumber
-	 * \param bNeedsLock Whether the function was called with the
-	 * audio engine locked already or it should do so itself.
+	 * \param nPat Sets #m_nSelectedPatternNumber
+	 * \param bNeedsLock Whether the function was called with the audio engine
+	 *   locked already or it should do so itself.
+	 * \param bForce When present, the `EVENT_SELECTED_PATTERN_CHANGED` will be
+	 *   triggered regardless of @a nPat. This is important when rearranging
+	 *   patterns in the song editor while the pattern editor is locked.
 	 */
-	void			setSelectedPatternNumber( int nPat, bool bNeedsLock = true );
+	void			setSelectedPatternNumber( int nPat,
+											  bool bNeedsLock = true,
+											  bool bForce = false );
 
 	/**
 	 * Updates the selected pattern to the one recorded note will be
@@ -342,12 +349,12 @@ void			previewSample( Sample *pSample );
 	bool			handleBeatCounter();
 	void			setBcOffsetAdjust();
 
-	/** Calling JackAudioDriver::releaseTimebaseMaster() directly from
+	/** Calling JackAudioDriver::releaseTimebaseControl() directly from
 	    the GUI*/
-	void			offJackMaster();
-	/** Calling JackAudioDriver::initTimebaseMaster() directly from
+	void			releaseJackTimebaseControl();
+	/** Calling JackAudioDriver::initTimebaseControl() directly from
 	    the GUI*/
-	void			onJackMaster();
+	void			initJackTimebaseControl();
 
 	void			__panic();
 	std::shared_ptr<Timeline>	getTimeline() const;
@@ -355,8 +362,17 @@ void			previewSample( Sample *pSample );
 	
 	//export management
 	bool			getIsExportSessionActive() const;
-	/** \return true on success.*/
-	bool			startExportSession( int rate, int depth );
+	/**
+	 * @param nSampleRate sample rate using which to export
+	 * @param nSampleDepth sample depth using which to export
+	 * @param fCompressionLevel Trades off audio quality against compression
+	 *   rate defined between 0.0 (maximum quality) and 1.0 (maximum
+	 *   compression).
+	 *
+	 * \return true on success
+	 * .*/
+	bool			startExportSession( int nSampleRate, int nSampleDepth,
+										double fCompressionLevel = 0.0 );
 	void			stopExportSession();
 	void			startExportSong( const QString& filename );
 	void			stopExportSong();
@@ -405,12 +421,12 @@ void			previewSample( Sample *pSample );
 	 * (#H2Core::Preferences::m_bJackTransportMode).
 	 */
 	bool			hasJackTransport() const;
-        float			getMasterBpm() const;
+        float			getJackTimebaseControllerBpm() const;
 
 	/**
-	 * Convenience function checking whether using the Timeline tempo
-	 * is set in the Preferences, Song::SONG_MODE is set, and there is
-	 * a JACK timebase master present.
+	 * Convenience function checking whether using the Timeline tempo is set in
+	 * the Preferences, Song::SONG_MODE is set, and there is an external JACK
+	 * Timebase controller (application) present.
 	 *
 	 * \return Whether the Timeline is used to determine the current speed.
 	 */
@@ -426,10 +442,10 @@ void			previewSample( Sample *pSample );
 	Tempo getTempoSource() const;
 	
 	/**
-	 * \return Whether we hasJackTransport() and there is an external
-	 * JACK timebase master broadcasting us tempo information and
-	 * making use disregard Hydrogen's Timeline information (see
-	 * #H2Core::JackAudioDriver::m_timebaseState).
+	 * \return Whether we hasJackTransport() and there is an external JACK
+	 *   Timebase controller broadcasting tempo information. If so, we disregard
+	 *   Hydrogen's Timeline information (see
+	 *   #H2Core::JackAudioDriver::m_timebaseState).
 	 */
 	JackAudioDriver::Timebase		getJackTimebaseState() const;
 	/** \return NsmClient::m_bUnderSessionManagement if NSM is
@@ -594,6 +610,9 @@ private:
 
 	SoundLibraryDatabase* m_pSoundLibraryDatabase;
 
+		/** Controls the instrument selection within a hihat group. */
+		int m_nHihatOpenness;
+
 	/** 
 	 * Constructor, entry point, and initialization of the
 	 * Hydrogen application.
@@ -685,6 +704,12 @@ inline int Hydrogen::getLastMidiEventParameter() const {
 inline void	Hydrogen::setLastMidiEventParameter( int nParam ) {
 	m_nLastMidiEventParameter = nParam;
 }
+inline int Hydrogen::getHihatOpenness() const {
+	return m_nHihatOpenness;
+}
+inline void Hydrogen::setHihatOpenness( int nValue ) {
+	m_nHihatOpenness = std::clamp( nValue, 0, 127 );
+}
 };
 
 #endif
