@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -164,10 +164,16 @@ QStringList AlsaAudioDriver::getDevices()
 			*sIOID = snd_device_name_get_hint( *pHint, "IOID");
 
 		if ( sIOID && QString( sIOID ) != "Output") {
+			free( (void *)sIOID );
+
+			if ( sName ) {
+				free( (void *)sName );
+			}
+
 			continue;
 		}
 
-		QString sDev = QString( sName );
+		const QString sDev = QString( sName );
 		if ( sName ) {
 			free( (void *)sName );
 		}
