@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -20,6 +20,16 @@
  *
  */
 
+#include <core/config.h>
+
+#include <QFile>
+
+#ifdef H2CORE_HAVE_QT6
+  #include <QStringConverter>
+#else
+  #include <QTextCodec>
+#endif
+
 #include <core/Lilipond/Lilypond.h>
 #include <core/Basics/Song.h>
 #include <core/Basics/PatternList.h>
@@ -84,32 +94,43 @@ void H2Core::LilyPond::extractData( const Song &song ) {
 }
 
 void H2Core::LilyPond::write( const QString &sFilename ) const {
-	std::ofstream file( sFilename.toLocal8Bit() );
-	if ( !file ) {
+	QFile file( sFilename );
+	if ( ! file.open( QIODevice::WriteOnly | QIODevice::Text ) ) {
+		ERRORLOG( QString( "Unable to open file [%1] for writing" )
+				  .arg( sFilename ) );
 		return;
 	}
 
-	file << sHeader;
-	file << "\\header {\n";
-	file << "    title = \"" << m_sName.toUtf8().constData() << "\"\n";
-	file << "    composer = \"" << m_sAuthor.toUtf8().constData() << "\"\n";
-	file << "    tagline = \"Generated by Hydrogen " H2CORE_VERSION "\"\n";
-	file << "}\n\n";
+	QTextStream stream( &file );
+#ifdef H2CORE_HAVE_QT6
+	stream.setEncoding( QStringConverter::Utf8 );
+#else
+	stream.setCodec( QTextCodec::codecForName( "UTF-8" ) );
+#endif
 
-	file << "\\score {\n";
-	file << "    \\new DrumStaff <<\n";
-	file << "        \\set DrumStaff.drumStyleTable = #(alist->hash-table "
+	stream << sHeader;
+	stream << "\\header {\n";
+	stream << "    title = \"" << m_sName << "\"\n";
+	stream << "    composer = \"" << m_sAuthor << "\"\n";
+	stream << "    tagline = \"Generated by Hydrogen " H2CORE_VERSION "\"\n";
+	stream << "}\n\n";
+
+	stream << "\\score {\n";
+	stream << "    \\new DrumStaff <<\n";
+	stream << "        \\set DrumStaff.drumStyleTable = #(alist->hash-table "
 	        "gmStyle)\n";
-	file << "        \\override Staff.TimeSignature #'style = #'() % Display "
+	stream << "        \\override Staff.TimeSignature #'style = #'() % Display "
 	        "4/4 signature\n";
-	file << "        \\set Staff.beamExceptions = #'()             % Beam "
+	stream << "        \\set Staff.beamExceptions = #'()             % Beam "
 	        "quavers two by two\n";
-	file << "        \\drummode {\n";
-	file << "            \\tempo 4 = " << static_cast<int>( m_fBPM ) << "\n\n";
-	writeMeasures( file );
-	file << "\n        }\n";
-	file << "    >>\n";
-	file << "}\n";
+	stream << "        \\drummode {\n";
+	stream << "            \\tempo 4 = " << static_cast<int>( m_fBPM ) << "\n\n";
+	writeMeasures( stream );
+	stream << "\n        }\n";
+	stream << "    >>\n";
+	stream << "}\n";
+
+	file.close();
 }
 
 void H2Core::LilyPond::addPatternList( const PatternList &list, notes_t &to ) {
@@ -142,7 +163,7 @@ void H2Core::LilyPond::addPattern( const Pattern &pattern, notes_t &notes ) {
 	}
 }
 
-void H2Core::LilyPond::writeMeasures( std::ofstream &stream ) const {
+void H2Core::LilyPond::writeMeasures( QTextStream &stream ) const {
 	unsigned nSignature = 0; ///< Numerator of the time signature
 	for ( unsigned nMeasure = 0; nMeasure < m_Measures.size(); nMeasure++ ) {
 		// Start a new measure
@@ -162,7 +183,7 @@ void H2Core::LilyPond::writeMeasures( std::ofstream &stream ) const {
 	}
 }
 
-void H2Core::LilyPond::writeUpper( std::ofstream &stream,
+void H2Core::LilyPond::writeUpper( QTextStream &stream,
                                    unsigned nMeasure ) const {
 	// On the upper voice, we want only cymbals and mid and high toms
 	std::vector<int> whiteList;
@@ -178,7 +199,7 @@ void H2Core::LilyPond::writeUpper( std::ofstream &stream,
 	writeVoice( stream, nMeasure, whiteList );
 }
 
-void H2Core::LilyPond::writeLower( std::ofstream &stream,
+void H2Core::LilyPond::writeLower( QTextStream &stream,
                                    unsigned nMeasure ) const {
 	std::vector<int> whiteList;
 	whiteList.push_back( 0 ); // Kick
@@ -198,7 +219,7 @@ static const char *const sNames[] = { "bd",   "wbl",   "sn",    "mar",
 	                                  "cymr", "cymc",  "cymra", "cymca" };
 
 ///< Write group of note (may also be a rest or a single note)
-static void writeNote( std::ofstream &stream, const std::vector<int> &notes ) {
+static void writeNote( QTextStream &stream, const std::vector<int> &notes ) {
 	switch ( notes.size() ) {
 	case 0: stream << "r"; break;
 	case 1: stream << sNames[ notes[ 0 ] ]; break;
@@ -212,7 +233,7 @@ static void writeNote( std::ofstream &stream, const std::vector<int> &notes ) {
 }
 
 ///< Write duration in LilyPond format, from number of 1/48th of a beat
-static void writeDuration( std::ofstream &stream, unsigned duration ) {
+static void writeDuration( QTextStream &stream, unsigned duration ) {
 	if ( 48 % duration == 0 ) {
 		// This is a basic note
 		if ( duration % 2 ) {
@@ -239,7 +260,7 @@ static void writeDuration( std::ofstream &stream, unsigned duration ) {
 	}
 }
 
-void H2Core::LilyPond::writeVoice( std::ofstream &stream,
+void H2Core::LilyPond::writeVoice( QTextStream &stream,
                                    unsigned nMeasure,
                                    const std::vector<int> &whiteList ) const {
 	stream << "                ";
