@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -44,6 +44,7 @@
 #include <core/IO/AlsaAudioDriver.h>
 
 #include <QDir>
+#include <QProcess>
 //#include <QApplication>
 
 namespace H2Core
@@ -118,6 +119,8 @@ Preferences::Preferences()
 	m_nExportModeIdx = 0;
 	m_nExportSampleRateIdx = 0;
 	m_nExportSampleDepthIdx = 0;
+	m_exportFormat = Filesystem::AudioFormat::Flac;
+	m_fExportCompressionLevel = 0.0;
 	m_bShowExportSongLicenseWarning = true;
 	m_bShowExportDrumkitLicenseWarning = true;
 	m_bShowExportDrumkitCopyleftWarning = true;
@@ -143,7 +146,7 @@ Preferences::Preferences()
 	m_patternCategories.push_back( QString("not_categorized") );
 
 	//___ audio engine properties ___
-	m_sAudioDriver = QString("Auto");
+	m_audioDriver = AudioDriver::Auto;
 	m_bUseMetronome = false;
 	m_fMetronomeVolume = 0.5;
 	m_nMaxNotes = 256;
@@ -213,9 +216,8 @@ Preferences::Preferences()
 	m_bJackConnectDefaults = true;
 	m_bJackTrackOuts = false;
 	m_bJackTimebaseEnabled = false;
-	m_bJackMasterMode = NO_JACK_TIME_MASTER;
+	m_bJackTimebaseMode = NO_JACK_TIMEBASE_CONTROL;
 	m_JackTrackOutputMode = JackTrackOutputMode::postFader;
-	m_JackBBTSync = JackBBTSyncMethod::constMeasure;
 
 	// OSC configuration
 	m_bOscServerEnabled = false;
@@ -255,7 +257,7 @@ Preferences::Preferences()
 	songEditorProperties.set(10, 10, 600, 250, true);
 	instrumentRackProperties.set(500, 20, 526, 437, true);
 	audioEngineInfoProperties.set(720, 120, 0, 0, false);
-	m_playlistDialogProperties.set(200, 300, 961, 397, false);
+	m_playlistDialogProperties.set(200, 300, 921, 703, false);
 	m_directorProperties.set(200, 300, 423, 377, false);
 	m_ladspaProperties[0].set(2, 20, 0, 0, false);
 	m_ladspaProperties[1].set(2, 20, 0, 0, false);
@@ -286,11 +288,6 @@ Preferences::~Preferences()
 	__instance = nullptr;
 }
 
-
-
-
-
-
 ///
 /// Load the preferences file
 ///
@@ -303,17 +300,11 @@ bool Preferences::loadPreferences( bool bGlobal )
 
 	bool bRecreate = false;	// configuration file must be recreated?
 
-	QString sPreferencesFilename;
-	const QString sPreferencesOverwritePath = Filesystem::getPreferencesOverwritePath();
-	if ( sPreferencesOverwritePath.isEmpty() ) {
-			sPreferencesFilename = ( bGlobal ? Filesystem::sys_config_path() : Filesystem::usr_config_path() );
-			INFOLOG( QString( "Loading preferences file (%1) [%2]" ).arg( bGlobal ? "SYS" : "USER" ).arg( sPreferencesFilename ) );
-	}
-	else {
-		sPreferencesFilename = sPreferencesOverwritePath;
-		INFOLOG( QString( "Loading preferences file [%1]" ).arg( sPreferencesFilename ) );
-	}
-	
+	QString sPreferencesFilename =
+		( bGlobal ? Filesystem::sys_config_path() : Filesystem::usr_config_path() );
+	INFOLOG( QString( "Loading preferences file [%1]" )
+			 .arg( sPreferencesFilename ) );
+
 	if ( ! Filesystem::file_readable( sPreferencesFilename, true ) ) {
 		if ( bGlobal ) {
 			ERRORLOG( QString( "Global preferences file [%1] is not readable!" )
@@ -330,7 +321,7 @@ bool Preferences::loadPreferences( bool bGlobal )
 		// Preferences is readable.
 
 		XMLDoc doc;
-		doc.read( sPreferencesFilename, nullptr, false );
+		doc.read( sPreferencesFilename, false );
 		XMLNode rootNode = doc.firstChildElement( "hydrogen_preferences" );
 
 		if ( !rootNode.isNull() ) {
@@ -435,17 +426,14 @@ bool Preferences::loadPreferences( bool bGlobal )
 				WARNINGLOG( "audio_engine node not found" );
 				bRecreate = true;
 			} else {
-				m_sAudioDriver = audioEngineNode.read_string( "audio_driver", m_sAudioDriver, false, false );
-				// Ensure compatibility will older versions of the
-				// files after capitalization in the GUI
-				// (2021-02-05). This can be dropped in releases >=
-				// 1.2
-				if ( m_sAudioDriver == "Jack" ) {
-					m_sAudioDriver = "JACK";
-				} else if ( m_sAudioDriver == "Oss" ) {
-					m_sAudioDriver = "OSS";
-				} else if ( m_sAudioDriver == "Alsa" ) {
-					m_sAudioDriver = "ALSA";
+				const QString sAudioDriver = audioEngineNode.read_string(
+					"audio_driver", Preferences::audioDriverToQString(
+						Preferences::AudioDriver::Auto ),
+					false, false );
+				m_audioDriver = parseAudioDriver( sAudioDriver );
+				if ( m_audioDriver == AudioDriver::None ) {
+					WARNINGLOG( "Falling back to 'Auto' audio driver" );
+					m_audioDriver = AudioDriver::Auto;
 				}
 				m_bUseMetronome = audioEngineNode.read_bool( "use_metronome", m_bUseMetronome, false, false );
 				m_fMetronomeVolume = audioEngineNode.read_float( "metronome_volume", 0.5f, false, false );
@@ -497,27 +485,17 @@ bool Preferences::loadPreferences( bool bGlobal )
 						m_bJackTransportMode = USE_JACK_TRANSPORT;
 					}
 
-					//jack time master
+					// We stick to the old Timebase strings (? why strings for a
+					// boolean option?) for backward and forward compatibility
+					// of old versions still in use.
 					m_bJackTimebaseEnabled = jackDriverNode.read_bool( "jack_timebase_enabled", false, false, false );
 					QString tmMode = jackDriverNode.read_string( "jack_transport_mode_master", "NO_JACK_TIME_MASTER", false, false );
 					if ( tmMode == "NO_JACK_TIME_MASTER" ) {
-						m_bJackMasterMode = NO_JACK_TIME_MASTER;
+						m_bJackTimebaseMode = NO_JACK_TIMEBASE_CONTROL;
 					} else if ( tmMode == "USE_JACK_TIME_MASTER" ) {
-						m_bJackMasterMode = USE_JACK_TIME_MASTER;
+						m_bJackTimebaseMode = USE_JACK_TIMEBASE_CONTROL;
 					}
 
-					int nBBTSync = jackDriverNode.read_int( "jack_bbt_sync", 0, false, false );
-					if ( nBBTSync == 0 ){
-						m_JackBBTSync = JackBBTSyncMethod::constMeasure;
-					} else if ( nBBTSync == 1 ) {
-						m_JackBBTSync = JackBBTSyncMethod::identicalBars;
-					} else {
-						WARNINGLOG( QString( "Unknown jack_bbt_sync value [%1]. Using JackBBTSyncMethod::constMeasure instead." )
-									.arg( nBBTSync ) );
-						m_JackBBTSync = JackBBTSyncMethod::constMeasure;
-					}
-					// ~ jack time master
-
 					m_bJackTrackOuts = jackDriverNode.read_bool( "jack_track_outs", m_bJackTrackOuts, false, false );
 					m_bJackConnectDefaults = jackDriverNode.read_bool( "jack_connect_defaults", m_bJackConnectDefaults, false, false );
 
@@ -552,15 +530,16 @@ bool Preferences::loadPreferences( bool bGlobal )
 					WARNINGLOG( "midi_driver node not found" );
 					bRecreate = true;
 				} else {
-					m_sMidiDriver = midiDriverNode.read_string( "driverName", "ALSA", false, false );
+					m_sMidiDriver = midiDriverNode.read_string(
+						"driverName", m_sMidiDriver, false, false );
 					// Ensure compatibility will older versions of the
 					// files after capitalization in the GUI
 					// (2021-02-05). This can be dropped in releases
 					// >= 1.2
-					if ( m_sAudioDriver == "JackMidi" ) {
-						m_sAudioDriver = "JACK-MIDI";
-					} else if ( m_sAudioDriver == "CoreMidi" ) {
-						m_sAudioDriver = "CoreMIDI";
+					if ( m_sMidiDriver == "JackMidi" ) {
+						m_sMidiDriver = "JACK-MIDI";
+					} else if ( m_sMidiDriver == "CoreMidi" ) {
+						m_sMidiDriver = "CoreMIDI";
 					}
 					m_sMidiPortName = midiDriverNode.read_string(
 						"port_name", Preferences::getNullMidiPort(), false, false );
@@ -637,6 +616,9 @@ bool Preferences::loadPreferences( bool bGlobal )
 				setSongEditorProperties( readWindowProperties( guiNode, "songEditor_properties", songEditorProperties ) );
 				setInstrumentRackProperties( readWindowProperties( guiNode, "instrumentRack_properties", instrumentRackProperties ) );
 				setAudioEngineInfoProperties( readWindowProperties( guiNode, "audioEngineInfo_properties", audioEngineInfoProperties ) );
+				// In order to be backward compatible we still call the XML node
+				// "playlistDialog". For some time we had playlistEditor and
+				// playlistDialog coexisting.
 				setPlaylistDialogProperties( readWindowProperties( guiNode, "playlistDialog_properties", m_playlistDialogProperties ) );
 				setDirectorProperties( readWindowProperties( guiNode, "director_properties", m_directorProperties ) );
 
@@ -659,7 +641,10 @@ bool Preferences::loadPreferences( bool bGlobal )
 				m_sLastExportThemeDirectory = guiNode.read_string( "lastExportThemeDirectory", QDir::homePath(), true, false, true );
 
 				//export dialog properties
-				m_nExportTemplateIdx = guiNode.read_int( "exportDialogTemplate", 0, false, false );
+				m_exportFormat = Filesystem::AudioFormatFromSuffix(
+					guiNode.read_string( "exportDialogFormat", "flac", true, true ) );
+				m_fExportCompressionLevel = guiNode.read_float(
+					"exportDialogCompressionLevel", 0.0, true, true );
 				m_nExportModeIdx = guiNode.read_int( "exportDialogMode", 0, false, false );
 				m_nExportSampleRateIdx = guiNode.read_int( "exportDialogSampleRate", 0, false, false );
 				m_nExportSampleDepthIdx = guiNode.read_int( "exportDialogSampleDepth", 0, false, false );
@@ -948,7 +933,8 @@ bool Preferences::savePreferences()
 	XMLNode audioEngineNode = rootNode.createNode( "audio_engine" );
 	{
 		// audio driver
-		audioEngineNode.write_string( "audio_driver", m_sAudioDriver );
+		audioEngineNode.write_string( "audio_driver",
+									  audioDriverToQString( m_audioDriver ) );
 
 		// use metronome
 		audioEngineNode.write_bool( "use_metronome", m_bUseMetronome );
@@ -983,7 +969,7 @@ bool Preferences::savePreferences()
 			jackDriverNode.write_string( "jack_port_name_1", m_sJackPortName1 );	// jack port name 1
 			jackDriverNode.write_string( "jack_port_name_2", m_sJackPortName2 );	// jack port name 2
 
-			// jack transport slave
+			// jack transport client
 			QString sMode;
 			if ( m_bJackTransportMode == NO_JACK_TRANSPORT ) {
 				sMode = "NO_JACK_TRANSPORT";
@@ -992,26 +978,18 @@ bool Preferences::savePreferences()
 			}
 			jackDriverNode.write_string( "jack_transport_mode", sMode );
 
-			//jack time master
 			jackDriverNode.write_bool( "jack_timebase_enabled", m_bJackTimebaseEnabled );
+			// We stick to the old Timebase strings (? why strings for a boolean
+			// option?) for backward and forward compatibility of old versions
+			// still in use.
 			QString tmMode;
-			if ( m_bJackMasterMode == NO_JACK_TIME_MASTER ) {
+			if ( m_bJackTimebaseMode == NO_JACK_TIMEBASE_CONTROL ) {
 				tmMode = "NO_JACK_TIME_MASTER";
-			} else if (  m_bJackMasterMode == USE_JACK_TIME_MASTER ) {
+			} else if (  m_bJackTimebaseMode == USE_JACK_TIMEBASE_CONTROL ) {
 				tmMode = "USE_JACK_TIME_MASTER";
 			}
 			jackDriverNode.write_string( "jack_transport_mode_master", tmMode );
 
-			int nBBTSync = 0;
-			if ( m_JackBBTSync == JackBBTSyncMethod::constMeasure ) {
-				nBBTSync = 0;
-			} else if ( m_JackBBTSync == JackBBTSyncMethod::identicalBars ) {
-				nBBTSync = 1;
-			}
-			jackDriverNode.write_int( "jack_bbt_sync", nBBTSync );
-			
-			// ~ jack time master
-
 			// jack default connection
 			jackDriverNode.write_bool( "jack_connect_defaults", m_bJackConnectDefaults );
 
@@ -1110,7 +1088,10 @@ bool Preferences::savePreferences()
 				
 		//ExportSongDialog
 		guiNode.write_int( "exportDialogMode", m_nExportModeIdx );
-		guiNode.write_int( "exportDialogTemplate", m_nExportTemplateIdx );
+		guiNode.write_string( "exportDialogFormat",
+							  Filesystem::AudioFormatToSuffix( m_exportFormat ) );
+		guiNode.write_float( "exportDialogCompressionLevel",
+							 m_fExportCompressionLevel );
 		guiNode.write_int( "exportDialogSampleRate",  m_nExportSampleRateIdx );
 		guiNode.write_int( "exportDialogSampleDepth", m_nExportSampleDepthIdx );
 		guiNode.write_bool( "showExportSongLicenseWarning", m_bShowExportSongLicenseWarning );
@@ -1237,6 +1218,213 @@ bool Preferences::savePreferences()
 	return doc.write( sPreferencesFilename );
 }
 
+Preferences::AudioDriver Preferences::parseAudioDriver( const QString& sDriver ) {
+	const QString s = QString( sDriver ).toLower();
+	if ( s == "auto" ) {
+		return AudioDriver::Auto;
+	}
+	else if ( s == "jack" || s == "jackaudio") {
+		return AudioDriver::Jack;
+	}
+	else if ( s == "oss" ) {
+		return AudioDriver::Oss;
+	}
+	else if ( s == "alsa" ) {
+		return AudioDriver::Alsa;
+	}
+	else if ( s == "pulseaudio" || s == "pulse" ) {
+		return AudioDriver::PulseAudio;
+	}
+	else if ( s == "coreaudio" || s == "core" ) {
+		return AudioDriver::CoreAudio;
+	}
+	else if ( s == "portaudio" || s == "port" ) {
+		return AudioDriver::PortAudio;
+	}
+	else {
+		if ( Logger::isAvailable() ) {
+			ERRORLOG( QString( "Unable to parse driver [%1]" ). arg( sDriver ) );
+		}
+		return AudioDriver::None;
+	}
+}
+
+QString Preferences::audioDriverToQString( const Preferences::AudioDriver& driver ) {
+	switch ( driver ) {
+	case AudioDriver::Auto:
+		return "Auto";
+	case AudioDriver::Jack:
+		return "JACK";
+	case AudioDriver::Oss:
+		return "OSS";
+	case AudioDriver::Alsa:
+		return "ALSA";
+	case AudioDriver::PulseAudio:
+		return "PulseAudio";
+	case AudioDriver::CoreAudio:
+		return "CoreAudio";
+	case AudioDriver::PortAudio:
+		return "PortAudio";
+	case AudioDriver::Disk:
+		return "Disk";
+	case AudioDriver::Fake:
+		return "Fake";
+	case AudioDriver::Null:
+		return "Null";
+	case AudioDriver::None:
+		return "nullptr";
+	default:
+		return "Unhandled driver type";
+	}
+}
+
+bool Preferences::checkJackSupport() {
+	// Check whether the Logger is already available.
+	const bool bUseLogger = Logger::isAvailable();
+
+#ifndef H2CORE_HAVE_JACK
+	if ( bUseLogger ) {
+		INFOLOG( "Hydrogen was compiled without JACK support." );
+	}
+	return false;
+#else
+  #ifndef H2CORE_HAVE_DYNAMIC_JACK_CHECK
+	if ( bUseLogger ) {
+		INFOLOG( "JACK support enabled." );
+	}
+	return true;
+  #else
+	/**
+	 * Calls @a sExecutable in a subprocess using the @a sOption CLI
+	 * option and reports the results.
+	 *
+	 * @return An empty string indicates, that the call exited with a
+	 *   code other than zero.
+	 */
+	auto checkExecutable = [&]( const QString& sExecutable,
+								const QString& sOption ) {
+		QProcess process;
+		process.start( sExecutable, QStringList( sOption ) );
+		process.waitForFinished( -1 );
+
+		if ( process.exitCode() != 0 ) {
+			return QString( "" );
+		}
+
+		QString sStdout = process.readAllStandardOutput();
+		if ( sStdout.isEmpty() ) {
+			return QString( "No output" );
+		}
+
+		return QString( sStdout.trimmed() );
+	};
+
+
+	bool bJackSupport = false;
+
+	// Classic JACK
+	QString sCapture = checkExecutable( "jackd", "--version" );
+	if ( ! sCapture.isEmpty() ) {
+		bJackSupport = true;
+		if ( bUseLogger ) {
+			INFOLOG( QString( "'jackd' of version [%1] found." )
+					 .arg( sCapture ) );
+		}
+	}
+
+	// JACK compiled with DBus support (maybe this one is packaged but
+	// the classical one isn't).
+	//
+	// `jackdbus` is supposed to be run by the DBus message daemon and
+	// does not have proper CLI options. But it does not fail by
+	// passing a `-h` either and this will serve for checking its
+	// presence.
+	sCapture = checkExecutable( "jackdbus", "-h" );
+	if ( ! sCapture.isEmpty() ) {
+		bJackSupport = true;
+		if ( bUseLogger ) {
+			INFOLOG( "'jackdbus' found." );
+		}
+	}
+
+	// Pipewire JACK interface
+	//
+	// `pw-jack` has no version query CLI option (yet). But showing
+	// the help will serve for checking its presence.
+	sCapture = checkExecutable( "pw-jack", "-h" );
+	if ( ! sCapture.isEmpty() ) {
+		bJackSupport = true;
+		if ( bUseLogger ) {
+			INFOLOG( "'pw-jack' found." );
+		}
+	}
+
+	if ( bUseLogger ) {
+		if ( bJackSupport ) {
+			INFOLOG( "Dynamic JACK discovery succeeded. JACK support enabled." );
+		}
+		else {
+			WARNINGLOG( "Dynamic JACK discovery failed. JACK support disabled." );
+		}
+	}
+
+	return bJackSupport;
+  #endif
+#endif
+}
+
+std::vector<Preferences::AudioDriver> Preferences::getSupportedAudioDrivers() {
+
+	std::vector<AudioDriver> drivers;
+
+	// We always do a fresh check. Maybe dynamical discovery will yield a
+	// different result this time.
+	bool bJackSupported = checkJackSupport();
+
+	// The order of the assigned drivers is important as Hydrogen uses
+	// it when trying different drivers in case "Auto" was selected.
+#if defined(WIN32)
+  #ifdef H2CORE_HAVE_PORTAUDIO
+	drivers.push_back( AudioDriver::PortAudio );
+  #endif
+	if ( bJackSupported ) {
+		drivers.push_back( AudioDriver::Jack );
+	}
+#elif defined(__APPLE__)
+  #ifdef H2CORE_HAVE_COREAUDIO
+	drivers.push_back( AudioDriver::CoreAudio );
+  #endif
+	if ( bJackSupported ) {
+		drivers.push_back( AudioDriver::Jack );
+	}
+  #ifdef H2CORE_HAVE_PULSEAUDIO
+	drivers.push_back( AudioDriver::PulseAudio );
+  #endif
+  #ifdef H2CORE_HAVE_PORTAUDIO
+	drivers.push_back( AudioDriver::PortAudio );
+  #endif
+#else /* Linux */
+	if ( bJackSupported ) {
+		drivers.push_back( AudioDriver::Jack );
+	}
+  #ifdef H2CORE_HAVE_PULSEAUDIO
+	drivers.push_back( AudioDriver::PulseAudio );
+  #endif
+  #ifdef H2CORE_HAVE_ALSA
+	drivers.push_back( AudioDriver::Alsa );
+  #endif
+  #ifdef H2CORE_HAVE_OSS
+	drivers.push_back( AudioDriver::Oss );
+  #endif
+  #ifdef H2CORE_HAVE_PORTAUDIO
+	drivers.push_back( AudioDriver::PortAudio );
+  #endif
+#endif
+
+	return drivers;
+}
+
+
 void Preferences::setMostRecentFX( QString FX_name )
 {
 	int pos = m_recentFX.indexOf( FX_name );
