@@ -1,6 +1,6 @@
 /*
  * Hydrogen
- * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -623,7 +623,7 @@ std::shared_ptr<Theme> Theme::importTheme( const QString& sPath ) {
 	INFOLOG( QString( "Importing theme to %1" ).arg( sPath ) );
 
 	XMLDoc doc;
-	if ( ! doc.read( sPath, nullptr, true ) ) {
+	if ( ! doc.read( sPath, true ) ) {
 		ERRORLOG( "Unable to load theme." );
 		return nullptr;
 	}
