@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -24,6 +24,7 @@
 #define H2C_VERSION_H
 
 #include <string>
+#include <QString>
 
 namespace H2Core {
 
@@ -35,6 +36,8 @@ std::string get_version();
  */
 bool version_older_than( int major, int minor, int patch );
 
+	QString getAboutText();
+
 };
 
 #endif // H2C_VERSION
