@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -74,7 +74,7 @@ void SampleWaveDisplay::paintEvent(QPaintEvent *ev)
 	}
 
 	QFont font;
-	font.setWeight( 63 );
+	font.setWeight( QFont::Bold );
 	painter.setFont( font );
 	painter.setPen( QColor( 255 , 255, 255, 200 ) );
 	painter.drawText( 0, 0, width(), 20, Qt::AlignCenter, m_sSampleName );
