@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -56,6 +56,7 @@
 #include <core/Basics/PatternList.h>
 #include <core/Basics/InstrumentList.h>
 
+#include "Widgets/AutomationPathView.h"
 #include "Widgets/InfoBar.h"
 
 #include <QtGui>
@@ -79,6 +80,7 @@ HydrogenApp::HydrogenApp( MainForm *pMainForm )
  , m_pDirector( nullptr )
  , m_nPreferencesUpdateTimeout( 100 )
  , m_bufferedChanges( H2Core::Preferences::Changes::None )
+ , m_pMainScrollArea( new QScrollArea )
 {
 	m_pInstance = this;
 
@@ -245,11 +247,11 @@ void HydrogenApp::setupSinglePanedInterface()
 	WindowProperties mainFormProp = pPref->getMainFormProperties();
 	setWindowProperties( m_pMainForm, mainFormProp, SetDefault & ~SetVisible );
 
-	m_pSplitter = new QSplitter( nullptr );
+	m_pSplitter = new QSplitter( m_pMainScrollArea );
 	m_pSplitter->setOrientation( Qt::Vertical );
 	m_pSplitter->setOpaqueResize( true );
 
-	m_pTab = new QTabWidget( nullptr );
+	m_pTab = new QTabWidget( m_pMainScrollArea );
 	m_pTab->setObjectName( "TabbedInterface" );
 
 	// SONG EDITOR
@@ -274,7 +276,7 @@ void HydrogenApp::setupSinglePanedInterface()
 	pSouthPanel->setObjectName( "SouthPanel" );
 	QHBoxLayout *pEditorHBox = new QHBoxLayout();
 	pEditorHBox->setSpacing( 5 );
-	pEditorHBox->setMargin( 0 );
+	pEditorHBox->setContentsMargins( 0, 0, 0, 0 );
 	pSouthPanel->setLayout( pEditorHBox );
 
 	// INSTRUMENT RACK
@@ -301,14 +303,13 @@ void HydrogenApp::setupSinglePanedInterface()
 	// PLayer control
 	m_pPlayerControl = new PlayerControl( nullptr );
 
-
 	QWidget *mainArea = new QWidget( m_pMainForm );	// this is the main widget
 	m_pMainForm->setCentralWidget( mainArea );
 
 	// LAYOUT!!
 	m_pMainVBox = new QVBoxLayout();
 	m_pMainVBox->setSpacing( 1 );
-	m_pMainVBox->setMargin( 0 );
+	m_pMainVBox->setContentsMargins( 0, 0, 0, 0 );
 	m_pMainVBox->addWidget( m_pPlayerControl );
 
 	m_pMainVBox->addSpacing( 3 );
@@ -317,13 +318,26 @@ void HydrogenApp::setupSinglePanedInterface()
 		m_pMainVBox->addWidget( m_pSplitter );
 	} else {
 		m_pMainVBox->addWidget( m_pTab );
-
 	}
 
 	mainArea->setLayout( m_pMainVBox );
 
+	mainArea->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Expanding );
+	mainArea->setMinimumSize( 1000,
+							  180 + // menu bar, margins etc.
+							  PlayerControl::m_nMinimumHeight +
+							  SongEditorPanel::m_nMinimumHeight +
+							  InstrumentRack::m_nMinimumHeight +
+							  SongEditorPositionRuler::m_nMinimumHeight +
+							  SongEditor::m_nMinimumHeight +
+							  AutomationPathView::m_nMinimumHeight );
 
+	m_pMainScrollArea->setVerticalScrollBarPolicy( Qt::ScrollBarAsNeeded );
+	m_pMainScrollArea->setHorizontalScrollBarPolicy( Qt::ScrollBarAsNeeded );
+	m_pMainScrollArea->setWidget( mainArea );
+	m_pMainScrollArea->setWidgetResizable( true );
 
+	m_pMainForm->setCentralWidget( m_pMainScrollArea );
 
 	// MIXER
 	m_pMixer = new Mixer(nullptr);
@@ -718,13 +732,25 @@ void HydrogenApp::onEventQueueTimer()
 
 	Event event;
 	while ( ( event = pQueue->pop_event() ).type != EVENT_NONE ) {
-		
+
+		if ( m_eventListenersToAdd.size() > 0 ||
+			 m_eventListenersToRemove.size() > 0 ) {
+			updateEventListeners();
+		}
+
 		// Provide the event to all EventListeners registered to
 		// HydrogenApp. By registering itself as EventListener and
 		// implementing at least on the methods used below a
 		// particular GUI component can react on specific events.
-		for (int i = 0; i < (int)m_EventListeners.size(); i++ ) {
-			EventListener *pListener = m_EventListeners[ i ];
+		for ( const auto& pListener : m_eventListeners ) {
+			if ( m_eventListenersToRemove.size() > 0 &&
+				 m_eventListenersToRemove.find( pListener ) !=
+				 m_eventListenersToRemove.end() ) {
+				// This listener was scheduled to be removed. Most probably the
+				// corresponding object is already destructed and we would risk
+				// a segfault when attempting to call its methods.
+				continue;
+			}
 
 			switch ( event.type ) {
 			case EVENT_STATE:
@@ -824,7 +850,7 @@ void HydrogenApp::onEventQueueTimer()
 				break;
 
 			case EVENT_JACK_TIMEBASE_STATE_CHANGED:
-				pListener->jackTimebaseStateChangedEvent();
+				pListener->jackTimebaseStateChangedEvent( event.value );
 				break;
 				
 			case EVENT_SONG_MODE_ACTIVATION:
@@ -952,23 +978,65 @@ void HydrogenApp::onEventQueueTimer()
 }
 
 
-void HydrogenApp::addEventListener( EventListener* pListener )
-{
-	if (pListener) {
-		m_EventListeners.push_back( pListener );
+void HydrogenApp::addEventListener( EventListener* pListener ) {
+	if ( pListener != nullptr ) {
+		m_eventListenersToAdd.insert( pListener );
+	}
+
+	if ( m_eventListenersToRemove.find( pListener ) !=
+		 m_eventListenersToRemove.end() ) {
+		// Listener was already scheduled to be removed. Last action wins.
+		for ( auto it = m_eventListenersToRemove.begin();
+			  it != m_eventListenersToRemove.end(); ) {
+			if ( *it == pListener ) {
+				it = m_eventListenersToRemove.erase( it );
+				break;
+			} else {
+				++it;
+			}
+		}
 	}
 }
 
+void HydrogenApp::removeEventListener( EventListener* pListener ) {
+	if ( pListener != nullptr ) {
+		m_eventListenersToRemove.insert( pListener );
+	}
 
-void HydrogenApp::removeEventListener( EventListener* pListener )
-{
-	for ( uint i = 0; i < m_EventListeners.size(); i++ ) {
-		if ( pListener == m_EventListeners[ i ] ) {
-			m_EventListeners.erase( m_EventListeners.begin() + i );
+	if ( m_eventListenersToAdd.find( pListener ) !=
+		 m_eventListenersToAdd.end() ) {
+		for ( auto it = m_eventListenersToAdd.begin();
+			  it != m_eventListenersToAdd.end(); ) {
+			if ( *it == pListener ) {
+				it = m_eventListenersToAdd.erase( it );
+				break;
+			} else {
+				++it;
+			}
 		}
 	}
 }
 
+void HydrogenApp::updateEventListeners() {
+	for ( const auto& ppEventListener : m_eventListenersToRemove ) {
+		for ( auto it = m_eventListeners.begin();
+			  it != m_eventListeners.end(); ) {
+			if ( *it == ppEventListener ) {
+				it = m_eventListeners.erase( it );
+				break;
+			} else {
+				++it;
+			}
+		}
+	}
+	m_eventListenersToRemove.clear();
+
+	for ( const auto& ppEventListener : m_eventListenersToAdd ) {
+		m_eventListeners.push_back( ppEventListener );
+	}
+	m_eventListenersToAdd.clear();
+}
+
 /**
  * Removes temporary files that were created
  * for undo'ing things.
