@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -44,32 +44,36 @@ using namespace H2Core;
 
 #include <cassert>
 
-#define MIXER_STRIP_WIDTH	56
-#define MASTERMIXER_STRIP_WIDTH	126
-
 Mixer::Mixer( QWidget* pParent )
  : QWidget( pParent )
 {
 	setWindowTitle( tr( "Mixer" ) );
-	setMaximumHeight( 284 );
-	setMinimumHeight( 284 );
-	setFixedHeight( 284 );
+
+	const int nMinimumFaderPanelWidth = MixerLine::nWidth * 4;
+	const int nFXFrameWidth = 213;
+	const int nFixedHeight = MasterMixerLine::nHeight;
+
+	setMinimumSize( nMinimumFaderPanelWidth +
+					nFXFrameWidth + MasterMixerLine::nWidth +
+					8, // Small margin for scrollbar
+					nFixedHeight + 6 );
 
 // fader Panel
 	m_pFaderHBox = new QHBoxLayout();
 	m_pFaderHBox->setSpacing( 0 );
-	m_pFaderHBox->setMargin( 0 );
+	m_pFaderHBox->setContentsMargins( 0, 0, 0, 0 );
 
 	m_pFaderPanel = new QWidget( nullptr );
-	m_pFaderPanel->resize( MIXER_STRIP_WIDTH * MAX_INSTRUMENTS, height() );
-
+	m_pFaderPanel->resize( MixerLine::nWidth * MAX_INSTRUMENTS, nFixedHeight );
+	m_pFaderPanel->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
+	m_pFaderPanel->setMinimumSize( nMinimumFaderPanelWidth, nFixedHeight );
+	m_pFaderPanel->setMaximumSize( 16777215, nFixedHeight );
 	m_pFaderPanel->setLayout( m_pFaderHBox );
 
 	m_pFaderScrollArea = new QScrollArea( nullptr );
 	m_pFaderScrollArea->setFrameShape( QFrame::NoFrame );
 	m_pFaderScrollArea->setVerticalScrollBarPolicy( Qt::ScrollBarAlwaysOff );
 	m_pFaderScrollArea->setHorizontalScrollBarPolicy( Qt::ScrollBarAlwaysOn );
-	m_pFaderScrollArea->setMinimumWidth( MIXER_STRIP_WIDTH * 4 );
 	m_pFaderScrollArea->setWidget( m_pFaderPanel );
 
 	for ( uint i = 0; i < MAX_INSTRUMENTS; ++i ) {
@@ -85,7 +89,7 @@ Mixer::Mixer( QWidget* pParent )
 #endif
 	m_pFXFrame = new PixmapWidget( nullptr );
 	m_pFXFrame->setObjectName( "MixerFXRack" );
-	m_pFXFrame->setFixedSize( 213, height() );
+	m_pFXFrame->setFixedSize( nFXFrameWidth, nFixedHeight );
 	m_pFXFrame->setPixmap( "/mixerPanel/background_FX.png" );
 	for (uint nFX = 0; nFX < MAX_FX; nFX++) {
 		m_pLadspaFXLine[nFX] = new LadspaFXMixerLine( m_pFXFrame );
@@ -116,7 +120,6 @@ Mixer::Mixer( QWidget* pParent )
 // Master frame
 	m_pMasterLine = new MasterMixerLine( nullptr );
 	m_pMasterLine->setObjectName( "MasterMixerLine" );
-	m_pMasterLine->move( 0, 0 );
 	connect( m_pMasterLine, SIGNAL( volumeChanged(MasterMixerLine*) ), this, SLOT( masterVolumeChanged(MasterMixerLine*) ) );
 	
 	m_pOpenMixerSettingsBtn = new Button( m_pMasterLine, QSize( 17, 17 ), Button::Type::Push, "cog.svg", "", false, QSize( 13, 13 ), tr( "Mixer Settings" ) );
@@ -147,12 +150,30 @@ Mixer::Mixer( QWidget* pParent )
 	// LAYOUT!
 	QHBoxLayout *pLayout = new QHBoxLayout();
 	pLayout->setSpacing( 0 );
-	pLayout->setMargin( 0 );
+	pLayout->setContentsMargins( 0, 0, 0, 0 );
 
 	pLayout->addWidget( m_pFaderScrollArea );
 	pLayout->addWidget( m_pFXFrame );
 	pLayout->addWidget( m_pMasterLine );
-	this->setLayout( pLayout );
+
+	QWidget* pMainWidget = new QWidget();
+	pMainWidget->setLayout( pLayout );
+	pMainWidget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
+	pMainWidget->setMinimumSize( nMinimumFaderPanelWidth +
+								 nFXFrameWidth + MasterMixerLine::nWidth, nFixedHeight );
+	pMainWidget->setMaximumSize( 16777215, nFixedHeight );
+
+	auto pMainScrollArea = new QScrollArea();
+	pMainScrollArea->setVerticalScrollBarPolicy( Qt::ScrollBarAsNeeded );
+	pMainScrollArea->setHorizontalScrollBarPolicy( Qt::ScrollBarAsNeeded );
+	pMainScrollArea->setWidget( pMainWidget );
+	pMainScrollArea->setWidgetResizable( true );
+
+	auto pMainLayout = new QHBoxLayout();
+	pMainLayout->setSpacing( 0 );
+	pMainLayout->setContentsMargins( 0, 0, 0, 0 );
+	pMainLayout->addWidget( pMainScrollArea );
+	setLayout( pMainLayout );
 
 
 	m_pUpdateTimer = new QTimer( this );
@@ -433,7 +454,7 @@ void Mixer::updateMixer()
 				delete m_pMixerLine[ nInstr ];
 				m_pMixerLine[ nInstr ] = nullptr;
 
-				int newWidth = MIXER_STRIP_WIDTH * ( nInstruments + nCompo );
+				int newWidth = MixerLine::nWidth * ( nInstruments + nCompo );
 				if ( m_pFaderPanel->width() != newWidth ) {
 					m_pFaderPanel->resize( newWidth, height() );
 				}
@@ -446,7 +467,7 @@ void Mixer::updateMixer()
 				m_pMixerLine[ nInstr ] = createMixerLine( nInstr );
 				m_pFaderHBox->insertWidget( nInstr, m_pMixerLine[ nInstr ] );
 
-				int newWidth = MIXER_STRIP_WIDTH * ( nInstruments + nCompo );
+				int newWidth = MixerLine::nWidth * ( nInstruments + nCompo );
 				if ( m_pFaderPanel->width() != newWidth ) {
 					m_pFaderPanel->resize( newWidth, height() );
 				}
@@ -532,7 +553,7 @@ void Mixer::updateMixer()
 			m_pComponentMixerLine[ pDrumkitComponent->get_id() ] = createComponentMixerLine( pDrumkitComponent->get_id() );
 			m_pFaderHBox->addWidget( m_pComponentMixerLine[ pDrumkitComponent->get_id() ] );
 
-			int newWidth = MIXER_STRIP_WIDTH * ( nInstruments + nCompo );
+			int newWidth = MixerLine::nWidth * ( nInstruments + nCompo );
 			if ( m_pFaderPanel->width() != newWidth ) {
 				m_pFaderPanel->resize( newWidth, height() );
 			}
@@ -607,7 +628,7 @@ void Mixer::updateMixer()
 			delete m_pComponentMixerLine[nCompoID];
 			m_pComponentMixerLine.erase( nCompoID );
 
-			int newWidth = MIXER_STRIP_WIDTH * ( nInstruments + nCompo );
+			int newWidth = MixerLine::nWidth * ( nInstruments + nCompo );
 			if ( m_pFaderPanel->width() != newWidth ) {
 				m_pFaderPanel->resize( newWidth, height() );
 			}
