@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -76,12 +76,12 @@ PlayerControl::PlayerControl(QWidget *parent)
 
 	QHBoxLayout *hbox = new QHBoxLayout();
 	hbox->setSpacing( 0 );
-	hbox->setMargin( 0 );
+	hbox->setContentsMargins( 0, 0, 0, 0 );
 	setLayout( hbox );
 
 // CONTROLS
 	PixmapWidget *pControlsPanel = new PixmapWidget( nullptr );
-	pControlsPanel->setFixedSize( 344, 43 );
+	pControlsPanel->setFixedSize( 344, m_nMinimumHeight );
 	pControlsPanel->setPixmap( "/playerControlPanel/background_Control.png" );
 	pControlsPanel->setObjectName( "ControlsPanel" );
 	hbox->addWidget( pControlsPanel );
@@ -223,7 +223,7 @@ PlayerControl::PlayerControl(QWidget *parent)
 
 	m_sBCOnOffBtnToolTip = tr("Toggle the BeatCounter Panel");
 	m_sBCOnOffBtnTimelineToolTip = tr( "Please deactivate the Timeline first in order to use the BeatCounter" );
-	m_sBCOnOffBtnJackTimebaseToolTip = tr( "In the presence of an external JACK Timebase master the BeatCounter can not be used" );
+	m_sBCOnOffBtnJackTimebaseToolTip = tr( "In the presence of an external JACK Timebase controller the BeatCounter can not be used" );
 	m_pBCOnOffBtn = new Button( pControlsBBTBConoffPanel, QSize( 13, 42 ),
 								Button::Type::Toggle, "",
 								pCommonStrings->getBeatCounterButton(), false,
@@ -319,7 +319,7 @@ PlayerControl::PlayerControl(QWidget *parent)
 // BPM
 	m_sLCDBPMSpinboxToolTip = tr("Alter the Playback Speed");
 	m_sLCDBPMSpinboxTimelineToolTip = tr( "While the Timeline is active this widget is in read-only mode and just displays the tempo set using the current Timeline position" );
-	m_sLCDBPMSpinboxJackTimebaseToolTip = tr( "In the presence of an external JACK Timebase master this widget just displays the tempo broadcasted by JACK" );
+	m_sLCDBPMSpinboxJackTimebaseToolTip = tr( "In the presence of an external JACK Timebase controller this widget just displays the tempo broadcasted by JACK" );
 
 	PixmapWidget *pBPMPanel = new PixmapWidget( nullptr );
 	pBPMPanel->setFixedSize( 145, 43 );
@@ -381,7 +381,7 @@ PlayerControl::PlayerControl(QWidget *parent)
 
 // JACK
 	PixmapWidget *pJackPanel = new PixmapWidget( nullptr );
-	pJackPanel->setFixedSize( 113, 43 );
+	pJackPanel->setFixedSize( 124, 43 );
 	pJackPanel->setPixmap( "/playerControlPanel/background_Jack.png" );
 	pJackPanel->setObjectName( "JackPanel" );
 	hbox->addWidget( pJackPanel );
@@ -408,49 +408,45 @@ PlayerControl::PlayerControl(QWidget *parent)
 	connect(m_pJackTransportBtn, SIGNAL( clicked() ), this, SLOT( jackTransportBtnClicked() ));
 	m_pJackTransportBtn->move( 3, 24 );
 
-	//jack time master
-	/*: Using the JACK Timebase Master functionality one of the
-	  connected programs can broadcast both speed and measure information to
-	  all other connected applications in order to have a more fine-grained
-	  transport control.*/
-	m_pJackMasterBtn = new Button( pJackPanel, QSize( 53, 16 ),
-								   Button::Type::Toggle, "",
-								   pCommonStrings->getJackMasterButton(), false,
-								   QSize(), pCommonStrings->getJackTBMMasterTooltip(),
-								   false, true );
-	m_pJackMasterBtn->setObjectName( "PlayerControlJackMasterButton" );
+	m_pJackTimebaseBtn = new Button(
+		pJackPanel, QSize( 64, 16 ), Button::Type::Toggle, "",
+		pCommonStrings->getJackTimebaseButton(), false, QSize(),
+		pCommonStrings->getJackTimebaseTooltip(), false, true );
+	m_pJackTimebaseBtn->setObjectName( "PlayerControlJackTimebaseButton" );
 	if ( ! m_pHydrogen->hasJackAudioDriver() ) {
-		m_pJackMasterBtn->hide();
+		m_pJackTimebaseBtn->hide();
 	}
 			
 	if ( pPref->m_bJackTimebaseEnabled ) {
 		if ( m_pHydrogen->hasJackTransport() ) {
 			if ( m_pHydrogen->getJackTimebaseState() ==
-				 JackAudioDriver::Timebase::Master ) {
-				m_pJackMasterBtn->setChecked( true );
+				 JackAudioDriver::Timebase::Controller ) {
+				m_pJackTimebaseBtn->setChecked( true );
 			}
 			else if ( m_pHydrogen->getJackTimebaseState() ==
-					  JackAudioDriver::Timebase::Slave ) {
-				m_pJackMasterBtn->setChecked( true );
-				m_pJackMasterBtn->setUseRedBackground( true );
-				m_pJackMasterBtn->setToolTip( pCommonStrings->getJackTBMSlaveTooltip() );
+					  JackAudioDriver::Timebase::Listener ) {
+				m_pJackTimebaseBtn->setChecked( true );
+				m_pJackTimebaseBtn->setUseRedBackground( true );
+				m_pJackTimebaseBtn->setToolTip(
+					pCommonStrings->getJackTimebaseListenerTooltip() );
 			}
 			else {
-				m_pJackMasterBtn->setChecked( false );
+				m_pJackTimebaseBtn->setChecked( false );
 			}
 		}
 		else {
-			m_pJackMasterBtn->setIsActive( false );
+			m_pJackTimebaseBtn->setIsActive( false );
 		}
 	}
 	else {
-		m_pJackMasterBtn->setIsActive( false );
-		m_pJackMasterBtn->setBaseToolTip( pCommonStrings->getJackMasterDisabledTooltip() );
+		m_pJackTimebaseBtn->setIsActive( false );
+		m_pJackTimebaseBtn->setBaseToolTip(
+			pCommonStrings->getJackTimebaseDisabledTooltip() );
 	}
 
-	connect(m_pJackMasterBtn, SIGNAL( clicked() ), this, SLOT( jackMasterBtnClicked() ));
-	m_pJackMasterBtn->move( 56, 24 );
-	// ~ jack time master
+	connect( m_pJackTimebaseBtn, SIGNAL( clicked() ), this,
+			SLOT( jackTimebaseBtnClicked() ) );
+	m_pJackTimebaseBtn->move( 56, 24 );
 
 	// CPU load widget
 	m_pCpuLoadWidget = new CpuLoadWidget( pJackPanel );
@@ -718,7 +714,8 @@ void PlayerControl::updateSongEvent( int nValue ) {
 		loopModeActivationEvent();
 		timelineActivationEvent();
 		jackTransportActivationEvent();
-		jackTimebaseStateChangedEvent();
+		jackTimebaseStateChangedEvent(
+			static_cast<int>(Hydrogen::get_instance()->getJackTimebaseState()) );
 		updatePlayerControl();
 	}
 }
@@ -912,8 +909,7 @@ void PlayerControl::jackTransportBtnClicked()
 }
 
 
-//jack time master
-void PlayerControl::jackMasterBtnClicked()
+void PlayerControl::jackTimebaseBtnClicked()
 {
 #ifdef H2CORE_HAVE_JACK
 	if ( !m_pHydrogen->hasJackTransport() ) {
@@ -924,15 +920,14 @@ void PlayerControl::jackMasterBtnClicked()
 	auto pPref = Preferences::get_instance();
 	auto pCoreActionController = Hydrogen::get_instance()->getCoreActionController();
 	
-	if ( pPref->m_bJackMasterMode == Preferences::USE_JACK_TIME_MASTER ) {
-		pCoreActionController->activateJackTimebaseMaster( false );
+	if ( pPref->m_bJackTimebaseMode == Preferences::USE_JACK_TIMEBASE_CONTROL ) {
+		pCoreActionController->activateJackTimebaseControl( false );
 	}
 	else {
-		pCoreActionController->activateJackTimebaseMaster( true );
+		pCoreActionController->activateJackTimebaseControl( true );
 	}
 #endif
 }
-// ~ jack time master
 
 void PlayerControl::fastForwardBtnClicked()
 {
@@ -1097,7 +1092,7 @@ void PlayerControl::tempoChangedEvent( int nValue )
 			QMessageBox::warning( this, "Hydrogen", tr("A tempo change via MIDI, OSC, BeatCounter, or TapTempo was detected. It will only be used after deactivating the Timeline and left of the first Tempo Marker when activating it again.") );
 		} else if ( pHydrogen->getTempoSource() ==
 					H2Core::Hydrogen::Tempo::Jack ) {
-			QMessageBox::warning( this, "Hydrogen", tr("A tempo change via MIDI, OSC, BeatCounter, or TapTempo was detected. It will only take effect when deactivating JACK BBT transport or making Hydrogen the Timebase master.") );
+			QMessageBox::warning( this, "Hydrogen", tr("A tempo change via MIDI, OSC, BeatCounter, or TapTempo was detected. It will only take effect when deactivating JACK Timebase support or making Hydrogen take Timebase control.") );
 		}
 	}
 }
@@ -1105,14 +1100,15 @@ void PlayerControl::tempoChangedEvent( int nValue )
 void PlayerControl::driverChangedEvent() {
 	if ( m_pHydrogen->hasJackAudioDriver() ) {
 		m_pJackTransportBtn->show();
-		m_pJackMasterBtn->show();
+		m_pJackTimebaseBtn->show();
 
-		jackTimebaseStateChangedEvent();
+		jackTimebaseStateChangedEvent(
+			static_cast<int>(Hydrogen::get_instance()->getJackTimebaseState() ));
 		jackTransportActivationEvent();
 	}
 	else {
 		m_pJackTransportBtn->hide();
-		m_pJackMasterBtn->hide();
+		m_pJackTimebaseBtn->hide();
 	}
 }
 
@@ -1129,8 +1125,9 @@ void PlayerControl::jackTransportActivationEvent( )
 		HydrogenApp::get_instance()->showStatusBarMessage( tr("JACK transport mode = On") );
 
 		if ( pPref->m_bJackTimebaseEnabled ) {
-			m_pJackMasterBtn->setIsActive( true );
-			jackTimebaseStateChangedEvent();
+			m_pJackTimebaseBtn->setIsActive( true );
+			jackTimebaseStateChangedEvent(
+				static_cast<int>(Hydrogen::get_instance()->getJackTimebaseState()) );
 		}
 	}
 	else {
@@ -1138,13 +1135,13 @@ void PlayerControl::jackTransportActivationEvent( )
 		if ( ! m_pJackTransportBtn->isDown() ) {
 			m_pJackTransportBtn->setChecked( false );
 		}
-		m_pJackMasterBtn->setChecked( false );
-		m_pJackMasterBtn->setIsActive( false );
+		m_pJackTimebaseBtn->setChecked( false );
+		m_pJackTimebaseBtn->setIsActive( false );
 		HydrogenApp::get_instance()->showStatusBarMessage( tr("JACK transport mode = Off") );
 	}
 }
 
-void PlayerControl::jackTimebaseStateChangedEvent()
+void PlayerControl::jackTimebaseStateChangedEvent( int nState )
 {
 	auto pPref = Preferences::get_instance();
 	if ( ! pPref->m_bJackTimebaseEnabled ) {
@@ -1152,40 +1149,40 @@ void PlayerControl::jackTimebaseStateChangedEvent()
 	}
 	
 	auto pCommonStrings = HydrogenApp::get_instance()->getCommonStrings();
-	auto pHydrogen = Hydrogen::get_instance();
 
 	QString sMessage = tr("JACK Timebase mode" ) + QString( " = " );
+	const auto state = JackAudioDriver::TimebaseFromInt( nState );
 	
-	switch( pHydrogen->getJackTimebaseState() ) {
-	case JackAudioDriver::Timebase::Master:
+	switch( state ) {
+	case JackAudioDriver::Timebase::Controller:
 
-		if ( ! m_pJackMasterBtn->isDown() ) {
-			m_pJackMasterBtn->setChecked( true );
+		if ( ! m_pJackTimebaseBtn->isDown() ) {
+			m_pJackTimebaseBtn->setChecked( true );
 		}
-		m_pJackMasterBtn->setUseRedBackground( false );
-		m_pJackMasterBtn->setToolTip( pCommonStrings->getJackTBMMasterTooltip() );
+		m_pJackTimebaseBtn->setUseRedBackground( false );
+		m_pJackTimebaseBtn->setToolTip( pCommonStrings->getJackTimebaseTooltip() );
 		
-		sMessage.append( "master" );
+		sMessage.append( "Controller" );
 		break;
 
-	case JackAudioDriver::Timebase::Slave:
+	case JackAudioDriver::Timebase::Listener:
 
-		if ( ! m_pJackMasterBtn->isDown() ) {
-			m_pJackMasterBtn->setChecked( true );
+		if ( ! m_pJackTimebaseBtn->isDown() ) {
+			m_pJackTimebaseBtn->setChecked( true );
 		}
-		m_pJackMasterBtn->setUseRedBackground( true );
-		m_pJackMasterBtn->setToolTip( pCommonStrings->getJackTBMSlaveTooltip() );
+		m_pJackTimebaseBtn->setUseRedBackground( true );
+		m_pJackTimebaseBtn->setToolTip( pCommonStrings->getJackTimebaseListenerTooltip() );
 		
-		sMessage.append( "slave" );
+		sMessage.append( "Listener" );
 		break;
 
 	default:
 
-		if ( ! m_pJackMasterBtn->isDown() ) {
-			m_pJackMasterBtn->setChecked( false );
+		if ( ! m_pJackTimebaseBtn->isDown() ) {
+			m_pJackTimebaseBtn->setChecked( false );
 		}		
-		m_pJackMasterBtn->setUseRedBackground( false );
-		m_pJackMasterBtn->setToolTip( pCommonStrings->getJackTBMMasterTooltip() );
+		m_pJackTimebaseBtn->setUseRedBackground( false );
+		m_pJackTimebaseBtn->setToolTip( pCommonStrings->getJackTimebaseTooltip() );
 
 		sMessage.append( pCommonStrings->getStatusOff() );
 	}
@@ -1200,20 +1197,22 @@ void PlayerControl::onPreferencesChanged( H2Core::Preferences::Changes changes )
 	if ( changes & H2Core::Preferences::Changes::AudioTab ) {
 		auto pPref = Preferences::get_instance();
 		auto pCommonStrings = HydrogenApp::get_instance()->getCommonStrings();
+		auto pHydrogen = Hydrogen::get_instance();
 
 		if ( pPref->m_bJackTimebaseEnabled ) {
-			if ( Hydrogen::get_instance()->hasJackTransport() ) {
-				m_pJackMasterBtn->setIsActive( true );
-				jackTimebaseStateChangedEvent();
+			if ( pHydrogen->hasJackTransport() ) {
+				m_pJackTimebaseBtn->setIsActive( true );
+				jackTimebaseStateChangedEvent(
+					static_cast<int>(pHydrogen->getJackTimebaseState()) );
 			}
 			else {
-				m_pJackMasterBtn->setToolTip( pCommonStrings->getJackTBMMasterTooltip() );
+				m_pJackTimebaseBtn->setToolTip( pCommonStrings->getJackTimebaseTooltip() );
 			}
 		}
 		else {
-			m_pJackMasterBtn->setChecked( false );
-			m_pJackMasterBtn->setIsActive( false );
-			m_pJackMasterBtn->setBaseToolTip( pCommonStrings->getJackMasterDisabledTooltip() );
+			m_pJackTimebaseBtn->setChecked( false );
+			m_pJackTimebaseBtn->setIsActive( false );
+			m_pJackTimebaseBtn->setBaseToolTip( pCommonStrings->getJackTimebaseDisabledTooltip() );
 		}
 	}
 }
