@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -56,7 +56,12 @@ SampleEditor::SampleEditor ( QWidget* pParent, int nSelectedComponent, int nSele
 		, Object ()
 {
 	setupUi ( this );
-	
+
+	// Show and enable maximize button. This is key when enlarging the
+	// application using a scaling factor and allows the OS to force its size
+	// beyond the minimum and make the scrollbars appear.
+	setWindowFlags( windowFlags() | Qt::CustomizeWindowHint |
+					Qt::WindowMinMaxButtonsHint );
 
 	m_pTimer = new QTimer(this);
 	connect(m_pTimer, SIGNAL(timeout()), this, SLOT(updateMainsamplePositionRuler()));
@@ -113,9 +118,6 @@ SampleEditor::SampleEditor ( QWidget* pParent, int nSelectedComponent, int nSele
 	openDisplays();
 	getAllFrameInfos();
 
-	adjustSize();
-	setFixedSize ( width(), height() );
-
 #ifndef H2CORE_HAVE_RUBBERBAND
 	if ( !Filesystem::file_executable( Preferences::get_instance()->m_rubberBandCLIexecutable , true /* silent */) ) {
 		RubberbandCframe->setDisabled ( true );
@@ -156,15 +158,14 @@ void SampleEditor::closeEvent(QCloseEvent *event)
 {
 	if ( !m_bSampleEditorClean ) {
 		auto pCommonStrings = HydrogenApp::get_instance()->getCommonStrings();
-		int err = QMessageBox::information( this, "Hydrogen",
-											pCommonStrings->getUnsavedChanges(),
-											pCommonStrings->getButtonOk(),
-											pCommonStrings->getButtonCancel(),
-											nullptr, 1 );
-		if ( err == 0 ) {
+		if ( QMessageBox::information(
+				 this, "Hydrogen", pCommonStrings->getUnsavedChanges(),
+				 QMessageBox::Ok | QMessageBox::Cancel,
+				 QMessageBox::Cancel ) == QMessageBox::Ok ) {
 			setClean();
 			accept();
-		} else {
+		}
+		else {
 			event->ignore();
 			return;
 		}
@@ -232,6 +233,15 @@ void SampleEditor::getAllFrameInfos()
 	m_bSampleIsModified = pSample->get_is_modified();
 	m_nSamplerate = pSample->get_sample_rate();
 	__loops = pSample->get_loops();
+
+	// Per default all loop frames will be set to zero by Hydrogen. But this is
+	// dangerous since just altering start or loop might move them beyond the
+	// end.
+	if ( __loops.start_frame == 0 &&
+		 __loops.loop_frame == 0 &&
+		 __loops.end_frame == 0 ) {
+		__loops.end_frame = pSample->get_frames();
+	}
 	__rubberband = pSample->get_rubberband();
 
 	if ( pSample->get_velocity_envelope()->size()==0 ) {
@@ -356,20 +366,18 @@ void SampleEditor::on_ClosePushButton_clicked()
 {
 	if ( !m_bSampleEditorClean ){
 		auto pCommonStrings = HydrogenApp::get_instance()->getCommonStrings();
-		int err = QMessageBox::information( this, "Hydrogen",
-											pCommonStrings->getUnsavedChanges(),
-											pCommonStrings->getButtonOk(),
-											pCommonStrings->getButtonCancel(),
-											nullptr, 1 );
-		if ( err == 0 ){
+		if ( QMessageBox::information(
+				 this, "Hydrogen", pCommonStrings->getUnsavedChanges(),
+				 QMessageBox::Ok | QMessageBox::Cancel,
+				 QMessageBox::Cancel ) == QMessageBox::Ok ) {
 			setClean();
 			accept();
-		}else
-		{
+		}
+		else {
 			return;
 		}
-	}else
-	{
+	}
+	else {
 		accept();
 	}
 }
@@ -392,18 +400,11 @@ void SampleEditor::on_PrevChangesPushButton_clicked()
 bool SampleEditor::getCloseQuestion()
 {
 	auto pCommonStrings = HydrogenApp::get_instance()->getCommonStrings();
-	
-	bool close = false;
-	int err = QMessageBox::information( this, "Hydrogen", 
-										pCommonStrings->getUnsavedChanges(),
-										pCommonStrings->getButtonOk(),
-										pCommonStrings->getButtonCancel(),
-										nullptr, 1 );
-	if ( err == 0 ) {
-		close = true;
-	}
 
-	return close;
+	return QMessageBox::information(
+		this, "Hydrogen", pCommonStrings->getUnsavedChanges(),
+		QMessageBox::Ok | QMessageBox::Cancel,
+		QMessageBox::Cancel ) == QMessageBox::Ok;
 }
 
 
@@ -656,34 +657,63 @@ void SampleEditor::on_PlayOrigPushButton_clicked()
 		testpTimer();
 		return;
 	}
+	auto pHydrogen = Hydrogen::get_instance();
+	auto tearDown = [&]() {
+		m_pMainSampleWaveDisplay->paintLocatorEvent(
+			StartFrameSpinBox->value() / m_divider + 24 , true);
+		m_pSampleAdjustView->setDetailSamplePosition(
+			__loops.start_frame, m_fZoomfactor , nullptr);
+		m_pTimer->start(40);	// update ruler at 25 fps
+		m_nRealtimeFrameEnd =
+			pHydrogen->getAudioEngine()->getRealtimeFrame() + m_nSlframes;
+		PlayOrigPushButton->setText( QString( "Stop") );
+	};
 
-	const int selectedlayer = InstrumentEditorPanel::get_instance()->getSelectedLayer();
-	std::shared_ptr<Song> pSong = Hydrogen::get_instance()->getSong();
-	auto pInstr = pSong->getInstrumentList()->get( Hydrogen::get_instance()->getSelectedInstrumentNumber() );
+	auto pSong = pHydrogen->getSong();
+	if ( pSong == nullptr ) {
+		tearDown();
+		return;
+	}
+
+	const int nSelectedlayer =
+		InstrumentEditorPanel::get_instance()->getSelectedLayer();
+	auto pInstr = pSong->getInstrumentList()->get(
+		pHydrogen->getSelectedInstrumentNumber() );
 	if ( pInstr == nullptr ) {
 		DEBUGLOG( "No instrument selected" );
+		tearDown();
 		return;
 	}
 
 	/*
-	 *preview_instrument deletes the last used preview instrument, therefore we have to construct a temporary
-	 *instrument. Otherwise pInstr would be deleted if consumed by preview_instrument.
+	 *preview_instrument deletes the last used preview instrument, therefore we
+	 *have to construct a temporary instrument. Otherwise pInstr would be
+	 *deleted if consumed by preview_instrument.
 	*/
-	auto pTmpInstrument = Instrument::load_instrument( pInstr->get_drumkit_path(), pInstr->get_name() );
-	auto pNewSample = Sample::load( pInstr->get_component( m_nSelectedComponent )->get_layer( selectedlayer )->get_sample()->get_filepath() );
-
-	if ( pNewSample != nullptr ){
-		int length = ( ( pNewSample->get_frames() / pNewSample->get_sample_rate() + 1) * 100 );
-		Hydrogen::get_instance()->getAudioEngine()->getSampler()->preview_instrument( pTmpInstrument );
-		Hydrogen::get_instance()->getAudioEngine()->getSampler()->preview_sample( pNewSample, length );
-		m_nSlframes = pNewSample->get_frames();
+	auto pTmpInstrument = Instrument::load_instrument(
+		pInstr->get_drumkit_path(), pInstr->get_name() );
+	if ( pTmpInstrument == nullptr ) {
+		ERRORLOG( QString( "Unable to load instrument [%1] from [%2]" )
+				  .arg( pInstr->get_name() ).arg( pInstr->get_drumkit_path() ) );
+		tearDown();
+		return;
+	}
+	const QString sSamplePath = pInstr->get_component( m_nSelectedComponent )
+		->get_layer( nSelectedlayer )->get_sample()->get_filepath();
+	auto pNewSample = Sample::load( sSamplePath );
+	if ( pNewSample == nullptr ) {
+		ERRORLOG( QString( "Unable to load sample from [%1]" )
+				  .arg( sSamplePath ) );
+		tearDown();
 	}
 
-	m_pMainSampleWaveDisplay->paintLocatorEvent( StartFrameSpinBox->value() / m_divider + 24 , true);
-	m_pSampleAdjustView->setDetailSamplePosition( __loops.start_frame, m_fZoomfactor , nullptr);
-	m_pTimer->start(40);	// update ruler at 25 fps
-	m_nRealtimeFrameEnd = Hydrogen::get_instance()->getAudioEngine()->getRealtimeFrame() + m_nSlframes;
-	PlayOrigPushButton->setText( QString( "Stop") );
+	const int nLength = ( pNewSample->get_frames() /
+						  pNewSample->get_sample_rate() + 1 ) * 100;
+	pHydrogen->getAudioEngine()->getSampler()->preview_instrument( pTmpInstrument );
+	pHydrogen->getAudioEngine()->getSampler()->preview_sample( pNewSample, nLength );
+	m_nSlframes = pNewSample->get_frames();
+
+	tearDown();
 }
 
 void SampleEditor::updateMainsamplePositionRuler()
@@ -853,8 +883,16 @@ void SampleEditor::valueChangedLoopCountSpinBox( int )
 		if ( ! m_bAdjusting ) on_PlayOrigPushButton_clicked();
 		return;
 	}
-	if ( m_nSlframes > Hydrogen::get_instance()->getAudioOutput()->getSampleRate() * 60 ){
-		Hydrogen::get_instance()->getAudioEngine()->getSampler()->stopPlayingNotes();
+
+	const auto pHydrogen = Hydrogen::get_instance();
+	const auto pAudioDriver = pHydrogen->getAudioOutput();
+	if ( pAudioDriver == nullptr ) {
+		ERRORLOG( "AudioDriver is not ready!" );
+		return;
+	}
+
+	if ( m_nSlframes > pAudioDriver->getSampleRate() * 60 ){
+		pHydrogen->getAudioEngine()->getSampler()->stopPlayingNotes();
 		m_pMainSampleWaveDisplay->paintLocatorEvent( -1 , false);
 		m_pTimer->stop();
 		m_bPlayButton = false;
@@ -862,7 +900,7 @@ void SampleEditor::valueChangedLoopCountSpinBox( int )
 	__loops.count = count; 
 	setUnclean();
 	setSamplelengthFrames();
-	if ( m_nSlframes > Hydrogen::get_instance()->getAudioOutput()->getSampleRate() * 60 * 30){ // >30 min
+	if ( m_nSlframes > pAudioDriver->getSampleRate() * 60 * 30){ // >30 min
 		LoopCountSpinBox->setMaximum(LoopCountSpinBox->value() -1);
 	}
 
