@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -28,6 +28,7 @@
 #include "../Widgets/FileDialog.h"
 #include "../HydrogenApp.h"
 #include "../InstrumentRack.h"
+#include "../CommonStrings.h"
 
 #include <core/H2Exception.h>
 #include <core/Preferences/Preferences.h>
@@ -42,6 +43,10 @@
 #include <QHeaderView>
 #include <QCryptographicHash>
 
+#ifndef H2CORE_HAVE_QT6
+  #include <QTextCodec>
+#endif
+
 #include <memory>
 
 const int max_redirects = 30;
@@ -50,7 +55,13 @@ SoundLibraryImportDialog::SoundLibraryImportDialog( QWidget* pParent, bool bOnli
  : QDialog( pParent )
 {
 	setupUi( this );
-	
+
+	// Show and enable maximize button. This is key when enlarging the
+	// application using a scaling factor and allows the OS to force its size
+	// beyond the minimum and make the scrollbars appear.
+	setWindowFlags( windowFlags() | Qt::CustomizeWindowHint |
+					Qt::WindowMinMaxButtonsHint );
+
 	setWindowTitle( tr( "Sound Library import" ) );
 
 	QStringList headers;
@@ -70,9 +81,6 @@ SoundLibraryImportDialog::SoundLibraryImportDialog( QWidget* pParent, bool bOnli
 
 	updateRepositoryCombo();
 
-	adjustSize();
-	setFixedSize( width(), height() );
-
 	if( bOnlineImport){
 		 tabWidget->setCurrentIndex( 0 );
 	} else {
@@ -713,19 +721,57 @@ void SoundLibraryImportDialog::on_BrowseBtn_clicked()
 void SoundLibraryImportDialog::on_InstallBtn_clicked()
 {
 	QApplication::setOverrideCursor(Qt::WaitCursor);
+
+	QString sError = tr( "An error occurred importing the SoundLibrary."  );
 	
 	try {
-		H2Core::Drumkit::install( SoundLibraryPathTxt->text() );
+		QString sImportedPath;
+		bool bEncodingIssues;
+		if ( ! H2Core::Drumkit::install(
+				 SoundLibraryPathTxt->text(), "", &sImportedPath,
+				 &bEncodingIssues, false ) ) {
+			QApplication::restoreOverrideCursor();
+			
+#ifndef H2CORE_HAVE_QT6
+			// Check whether encoding might be the problem in here.
+			auto pCodec = QTextCodec::codecForLocale();
+			if ( ! pCodec->canEncode( SoundLibraryPathTxt->text() ) ) {
+				QMessageBox::critical(
+					this, "Hydrogen", QString( "%1\n\n%2\n\n%3: [%4]" )
+					.arg( sError ).arg( SoundLibraryPathTxt->text() )
+					.arg( HydrogenApp::get_instance()->getCommonStrings()
+						  ->getEncodingError() )
+					.arg( QString( pCodec->name() ) ) );
+			}
+			else {
+				QMessageBox::critical( this, "Hydrogen", sError );
+			}
+#else
+		QMessageBox::critical( this, "Hydrogen", sError );
+#endif
+
+			return;
+		}
+
 		// update the drumkit list
 		H2Core::Hydrogen::get_instance()->getSoundLibraryDatabase()->update();
 		QApplication::restoreOverrideCursor();
-		QMessageBox::information( this, "Hydrogen",
-								  QString( tr( "SoundLibrary imported in %1" )
-										   .arg( H2Core::Filesystem::usr_data_path() )  ) );
+		if ( ! bEncodingIssues ) {
+			QMessageBox::information( this, "Hydrogen",
+									  QString( tr( "SoundLibrary imported in %1" )
+											   .arg( sImportedPath ) ) );
+		}
+		else {
+			QMessageBox::warning(
+				this, "Hydrogen",
+				QString( tr( "SoundLibrary imported in %1" )
+						 .arg( sImportedPath ) )
+				.append( tr( "\nBut there were encoding issues.\n\nPlease set your system's locale to UTF-8!" ) ) );
+		}
 	}
 	catch( H2Core::H2Exception ex ) {
 		QApplication::restoreOverrideCursor();
-		QMessageBox::warning( this, "Hydrogen", tr( "An error occurred importing the SoundLibrary."  ) );
+		QMessageBox::warning( this, "Hydrogen", sError );
 	}
 }
 
