@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2023 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -635,35 +635,6 @@ private:
 	std::vector< H2Core::Note *> m_overwritten;
 };
 
-
-/** \ingroup docGUI*/
-class SE_addNoteOffAction : public QUndoCommand
-{
-public:
-	SE_addNoteOffAction( int nColumn, int nRow, int selectedPatternNumber, bool isDelete ){
-		setText( QObject::tr( "Add NOTE_OFF note ( %1, %2 )" ).arg( nColumn ).arg( nRow ) );
-		__nColumn = nColumn;
-		__nRow = nRow;
-		__selectedPatternNumber = selectedPatternNumber;
-		__isDelete = isDelete;
-	}
-	virtual void undo()
-	{
-		HydrogenApp* h2app = HydrogenApp::get_instance();
-		h2app->getPatternEditorPanel()->getDrumPatternEditor()->addOrDeleteNoteAction( __nColumn, __nRow, __selectedPatternNumber, -1, 0.8f, 0.f, 0.0, 0, 0, 1.0f, false, false, false, true, !__isDelete ) ;
-	}
-	virtual void redo()
-	{
-		HydrogenApp* h2app = HydrogenApp::get_instance();
-		h2app->getPatternEditorPanel()->getDrumPatternEditor()->addOrDeleteNoteAction( __nColumn, __nRow, __selectedPatternNumber, -1, 0.8f, 0.f, 0.0, 0, 0, 1.0f, false, false, false, true, __isDelete );
-	}
-private:
-	int __nColumn;
-	int __nRow;
-	int __selectedPatternNumber;
-	bool __isDelete;
-};
-
 /** \ingroup docGUI*/
 class SE_moveNoteAction : public QUndoCommand
 {
@@ -884,11 +855,10 @@ public:
 		HydrogenApp* h2app = HydrogenApp::get_instance();
 		h2app->getPatternEditorPanel()->getDrumPatternEditor()->functionClearNotesUndoAction( __noteList, __nSelectedInstrument, __selectedPatternNumber );
 	}
-	virtual void redo()
-	{
-		//qDebug() << "clear note sequence Redo " ;
-		HydrogenApp* h2app = HydrogenApp::get_instance();
-		h2app->getPatternEditorPanel()->getDrumPatternEditor()->functionClearNotesRedoAction( __nSelectedInstrument, __selectedPatternNumber );
+	virtual void redo() {
+		H2Core::Hydrogen::get_instance()->getCoreActionController()
+			->clearInstrumentInPattern( __nSelectedInstrument,
+										__selectedPatternNumber );
 	}
 private:
 	std::list< H2Core::Note* > __noteList;
