@@ -1,7 +1,7 @@
 /*
  * Hydrogen
  * Copyright(c) 2002-2008 by Alex >Comix< Cominu [comix@users.sourceforge.net]
- * Copyright(c) 2008-2024 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
+ * Copyright(c) 2008-2025 The hydrogen development team [hydrogen-devel@lists.sourceforge.net]
  *
  * http://www.hydrogen-music.org
  *
@@ -25,7 +25,7 @@
 
 #include <QString>
 #include <cassert>
-#include <core/Smf/SMF.h>
+#include <core/SMF/SMF.h>
 
 class TestHelper {
 	static TestHelper*	m_pInstance;
@@ -60,10 +60,15 @@ class TestHelper {
 	 * \param sFileName Output file name
 	 * @param nSampleRate sample rate using which to export
 	 * @param nSampleDepth sample depth using which to export
+	 * @param fCompressionLevel Trades off audio quality against compression
+	 *   rate defined between 0.0 (maximum quality) and 1.0 (maximum
+	 *   compression).
 	 */
 	static void exportSong( const QString& sSongFile,
 							const QString& sFileName,
-							int nSampleRate = 44100, int nSampleDepth = 16 );
+							int nSampleRate = 44100,
+							int nSampleDepth = 16,
+							double fCompressionLevel = 0.0 );
 	/**
 	 * Export the current song within Hydrogen to audio file @a sFileName;
 	 *
